<script setup>
import AppLayoutCity from "@/Layouts/AppLayoutCity.vue";
import Welcome from "@/Jetstream/Welcome.vue";
import mapboxgl from "mapbox-gl";
import "mapbox-gl/dist/mapbox-gl.css";
import VuePapaParse from "vue-papa-parse";
import { conditionalExpression, templateElement } from "@babel/types";
import { Link } from '@inertiajs/inertia-vue3';
import Dk_Notification from '@/components/dk_Notification.vue';
import { axiosApiCalls } from '../../../public/libs/datakorumJS';

</script>
<style>
*,
::before,
::after {
  --tw-ring-color: rgb(59 130 246 / 0) !important;
}

.map-overlay {
  position: absolute;
  width: 30px;
  top: 4.5rem;
  left: 1rem;
  padding: 0px;
}

.map-overlay .map-overlay-inner {
  background-color: rgba(0, 42, 80, 0);
  /*box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);*/
  border-radius: 8px;
  padding: 0px;
  margin-bottom: 10px;
  height: 60vh;
}

/**/
#container {
  max-width: 660px;
  margin: auto;
  height: 400px;
  margin: 0 auto;
}

.percents {
  font-weight: bold;
  font-size: 22px;
}

.divide {
  width: 100%;
  margin: 0px;
  border: 0.5px solid black;
}

</style>
<template>

  <!--<DK_Alert
      :visibility="ComponentVisibility" @close="hideComponent" title="Email not associated with any account"
      message="The email address provided by the user is not associated with any existing account. Please double-check that it is spelled correctly or contact our technical support for assistance.">
  </DK_Alert>-->

  <AppLayoutCity title="Global View">

    <Dk_Notification id="notification" :corner="true" :title="this.Notification_Title" :message="this.Notification_Message" :hover="this.Show_Notification"></Dk_Notification>

    <div class="baltoro-container-2 overflow-x-hidden w-48 h-fit text-white-snow box-border z-50 ml-4 mt-16 left-0 top-0 fixed" v-if="this.screen_width >= 768"
      id="screendevices">
      <input type="text" v-on:keyup="searchfilter()" :placeholder="$t('general.Search')"
        class="focus-visible:outline-0 bg-cyan-opa text-md text-center text-white-snow font-normal w-48" id="searchbar" />
      <ul class="text-xs text-center text-gray-light max-h-48 h-fit overflow-y-scroll" id="device_list"></ul>
    </div>

    <div id="legend"
      class="baltoro-container-2 w-fit h-fit box-border z-50 ml-4 mb-11 left-0 bottom-0 fixed flex flex-col py-1 px-2 text-gray-light text-sm">
    </div>

    <!-- START RIGHT SIDEMENU -->
    <div id="sidemenu" style="height: calc(100vh - 4.75rem) !important" v-if="this.screen_width >= 768"
      class="cursor-default bg-transparent content-start grid grid-cols-2 gap-2 box-border duration-200 overflow-y-scroll overflow-x-hidden over z-50 pb-7 right-0 top-13 w-0 fixed float-left">
      <div class="col-span-2" id="sidemenu_content_before"></div>

      <!-- CHARTS SEPARADOS-->
      <div v-for="(Name, index) in this.charts" :key="index" :id="'chart_box_' + index"
        class="baltoro-container-2 w-full h-fit duration-200" v-bind:class="{
            'hidden': this.charts[index] == undefined,
            'col-span-2 px-4': !this.charts_size.includes(index),
            'col-span-1': this.charts_size.includes(index),
          }">
        <div class="border-0" v-bind:class="{
            'flex flex-col items-center px-3': this.charts_size.includes(index),
          }">
          <p v-if="this.charts[index] != undefined" class="text-xs text-center text-cyan-text font-normal my-1 whitespace-nowrap">
            {{ this.charts[index].name }}
          </p>
          <div :id="'chart_' + index" class="float-left" v-bind:class="{
              'w-full h-[3.6rem]':
                !this.charts_size.includes(index) &&
                !this.charts_text.includes(index) &&
                this.charts_height.includes(index),
              'w-full h-24':
                !this.charts_size.includes(index) &&
                !this.charts_text.includes(index) &&
                !this.charts_height.includes(index),
              'h-fit w-full':
                this.charts_size.includes(index) &&
                !this.charts_text.includes(index),
              'h-0':
              this.charts_null.includes(index),
              '':
                this.charts_text.includes(index),
            }"></div>
          <div class="w-full" :id="'chart_text_' + index"></div>
        </div>
      </div>

      <div class="col-span-2" id="sidemenu_content_after"></div>
    </div>
    <!-- END RIGHT SIDEMENU -->

    <div id="sidemenu_default" v-if="this.screen_width >= 768"
      class="cursor-default bg-transparent content-start grid grid-cols-2 gap-4 box-border duration-200 overflow-y-scroll overflow-x-hidden over z-50 px-2 right-0 top-13 w-[18rem] fixed float-left">

      <div class="col-span-2" id="sidemenu_default_content_before"></div>

      <!-- CHARTS SEPARADOS-->
      <div v-for="(Name, index) in this.default_charts" :key="index" :id="'default_chart_box_' + index"
        class="baltoro-container-2 w-full h-fit duration-200" v-bind:class="{
            'hidden': this.default_charts[index] == undefined,
            'col-span-2 px-4': !this.default_charts_size.includes(index),
            'col-span-1': this.default_charts_size.includes(index),
          }">
        <div class="border-0" v-bind:class="{
            'flex flex-col items-center': this.default_charts_size.includes(index),
          }">
          <p v-if="this.default_charts[index] != undefined" class="text-xs text-center text-cyan-text font-normal my-1">
            {{ this.default_charts[index].name }}
          </p>
          <div :id="'default_chart_' + index" class="float-left" v-bind:class="{
              'w-full h-[3.6rem]':
                !this.default_charts_size.includes(index) &&
                !this.default_charts_text.includes(index) &&
                this.default_charts_height.includes(index),
              'w-full h-24':
                !this.default_charts_size.includes(index) &&
                !this.default_charts_text.includes(index) &&
                !this.default_charts_height.includes(index),
              'h-32':
                this.default_charts_size.includes(index) &&
                !this.default_charts_text.includes(index),
              '':
                this.default_charts_text.includes(index),
            }"></div>
          <div :id="'default_chart_text_' + index"></div>
        </div>
      </div>

      <div class="col-span-2" id="sidemenu_default_content_after"></div>

    </div>

    <!-- map -->
    <div style="height: calc(100vh - 4.75rem)" id="map" />

    <!-- overlay -->
    <div class="map-overlay">
      <div class="map-overlay-inner"></div>
    </div>
    <div v-for="(gateway, index) in this.gateways" :key="index" class="h-0 w-0 invisible">
      <Link :id="'LinkProfile_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('profile', { id: gateway.id })"></Link>
      <Link :id="'LinkReadings_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('cityReadings', { idDevice: gateway.id })"></Link>
      <div v-if="gateway.MODBUS_device_type == 6 || gateway.MODBUS_device_type == 7">
        <Link :id="'LinkState_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('vacon', { id: gateway.id, page: 1 })"></Link>
        <Link :id="'LinkParameters_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('vacon', { id: gateway.id, page: 0 })"></Link>
      </div>
      <div v-if="gateway.MODBUS_device_type == 23">
        <Link :id="'LinkRTDashboard_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('rtDashboard', { id: gateway.id })"></Link>
      </div>
      <div v-if="gateway.MODBUS_device_type == 34">
        <Link :id="'LinkOptidrive_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('optidrive', { id: gateway.id })"></Link>
      </div>
      <div v-if="gateway.MODBUS_device_type == 31">
        <Link :id="'LinkWilo_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('wilo', { id: gateway.id })"></Link>
      </div>
      <div v-else-if="gateway.solutions_id == 8">
        <Link :id="'LinkMeteo_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('meteo', { id: gateway.id })"></Link>
      </div>
      <div v-else-if="gateway.solutions_id == 9">
        <Link :id="'LinkValve_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('valve', { id: gateway.id })"></Link>
      </div>
      <div v-else-if="gateway.solutions_id == 11">
        <Link :id="'LinkUllastres_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('ullastres', { id: gateway.id })"></Link>
      </div>
    </div>
  </AppLayoutCity>
</template>

<script>
export default {
  data() {
    return {
      screen_width: 0,
      Chart: {},
      geojsonobj: {},
      id_open: undefined,
      charts: new Array(15),
      charts_size: new Array(),
      charts_null: new Array(),
      charts_height: new Array(),
      charts_text: new Array(),
      default_charts: new Array(15),
      default_charts_size: new Array(),
      default_charts_height: new Array(),
      default_charts_null: new Array(),
      default_charts_text: new Array(),
      showlist: false,
      Show_Notification: false,
      Notification_Title: '',
      Notification_Message: '',
      week_day: new Date().getDay(),
      select_mode: 0,
      map: undefined,
      sidemenu_iddevice: null,

      getLastData: (idDevice) => {
        return axiosApiCalls.methodGet('/gateway/values/last/'+this.user.id+'/'+idDevice+'/0', this.user.token, false)
          .then((response) => {
              return response;
            })
      },

      getData: (idDevice) => {
        return axiosApiCalls.methodGet('/gateway/values/last/'+this.user.id+'/'+idDevice+'/1440', this.user.token)
          .then((response) => {
              return response;
            })
      },

      formatDate: (date) => {

        date = new Date(date);

        return (
            date.getUTCFullYear() + "-" +
            ((date.getUTCMonth() + 1 + '').length < 2 ? "0" + (date.getUTCMonth() + 1) : (date.getUTCMonth() + 1)) + "-" +
            ((date.getUTCDate() + '').length < 2 ? "0" + date.getUTCDate() : date.getUTCDate()) + " " +
            ((date.getUTCHours() + '').length < 2 ? "0" + date.getUTCHours() : date.getUTCHours()) + ":" +
            ((date.getUTCMinutes() + '').length < 2 ? "0" + date.getUTCMinutes() : date.getUTCMinutes()) + ":" +
            ((date.getUTCSeconds() + '').length < 2 ? "0" + date.getUTCSeconds() : date.getUTCSeconds())
        );
      },

      formatTime: (date_state_change, diff, short, hoursOffset) => {

        let duration_ms;
        if (diff == true) {
          let date = new Date(date_state_change);

          if (date.getHours() == 0 && date.getMinutes() == 0 && date.getSeconds() == 0) {
            return 'yesterday'; // Cambiarlo
          }

          var newDate = new Date();

          if(typeof hoursOffset != "undefined"){
            newDate = new Date(newDate.getTime() + ((hoursOffset + newDate.getTimezoneOffset() / 60) * 60 * 60 * 1000));
          }

          duration_ms = newDate - date;

        } else {
          duration_ms = date_state_change * 1000;
        }

        const diff_D = Math.floor(duration_ms / (1000 * 60 * 60 * 24));
        const diff_H = Math.floor(
          (duration_ms % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60)
        );
        const diff_M = Math.floor(
          (duration_ms % (1000 * 60 * 60)) / (1000 * 60)
        );

        let duration = "";

        if (this.user.language == "es" && diff) {
          duration = this.$t("general.for") + " ";
        }

        if (diff_D > 0) {
          duration += this.$t("general.morethan") + " " + diff_D + " ";
          if (diff_D > 1) {
            duration += this.$t("general.days");
          } else {
            duration += this.$t("general.day");
          }
        } else {
          if (diff_H > 0) {
            duration += diff_H;
            if (diff_M > 0) {
              duration += this.$t("general.h");
            } else if (diff_H > 1) {
              duration += " " + this.$t("general.hours");
            } else {
              duration += " " + this.$t("general.hour");
            }

            /*if(diff_M != 0){
                duration += ' '
              }*/

            if (diff_M != 0) {
              duration += " " + this.$t("general.and") + " ";
            }
          }

          if (diff_M > 0) {
            duration += diff_M;
            if (diff_H != 0 && diff_M > 1) {
              duration += this.$t("general.mins");
            } else if (diff_H != 0 && diff_M == 1) {
              duration += this.$t("general.min");
            } else if (diff_M > 1) {
              duration += " " + this.$t("general.minutes");
            } else {
              duration += " " + this.$t("general.minute");
            }
          } else if (short) {
            return this.$t("general.now");
          }
        }

        if (this.user.language == "en" && diff) {
          duration += " " + this.$t("general.ago");
        }

        if(duration === ''){
          duration = '0 ' + this.$t("general.min");
        }

        return duration;
      },
    };
  },

  props: ["gateways", "solutions", "location", "alerts_log", "data", "user"],

  beforeCreate() {

    localStorage.setItem("Back", "map");

    this.$i18n.locale = this.user.language;
    localStorage.setItem("Language", this.$i18n.locale);

    localStorage.setItem("Alerts", 0);

    for (var index = 0; index < this.alerts_log.length; index++) {
      if (this.alerts_log[index].read == 0) {
        localStorage.setItem("Alerts", +localStorage.getItem("Alerts") + 1);
      }
    }

    if (+localStorage.getItem("Alerts") > 9) {
      localStorage.setItem("Alerts", "9+");
    }

  },

  beforeMount() {
      this.screen_width = window.screen.width;
  },

  mounted() {
    window.emitter.emit('changeLoaderStatus', false, 'orbit')

    let formatDate = this.formatDate;

    this.Chart = {
      chart: {
        backgroundColor: "transparent",
        spacingTop: 15,
      },
      accessibility: {
        enabled: false,
      },
      title: {
        text: "",
      },
      xAxis: {
        visible: true,
        labels: {
          style: {
            color: "rgba(200, 200, 200, 0.9)",
            opacity: 0.7,
          },
        },
        type: "datetime",
        dateTimeLabelFormats: {
          //millisecond: '%H:%M:%S.%L',
          //second: '%H:%M:%S',
          //minute: '%H:%M',
          hour: "%H:%M",
          day: "%m-%d",
          week: "%m-%d",
          month: "%Y-%m",
          year: "%Y",
        },
      },
      yAxis: {
        title: {
          text: "",
        },
        labels: {
          style: {
            color: "rgba(200, 200, 200, 0.9)",
          },
        },
        visible: true,
      },
      exporting: {
        enabled: false,
      },
      credits: {
        enabled: false,
      },
      legend: {
        enabled: false,
      },
      tooltip: {
        shared: true,
        split: false,
        backgroundColor: "rgba(7,82,103,0.95)",
        borderRadius: 12,
        borderWidth: 1,
        borderColor: "rgba(250, 250, 250, 0.65)",
        padding: 6,

        formatter: function () {
          var s =
            "<b>" +
            '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
            formatDate(this.x) +
            "</p><br/>";
          $.each(this.points, function () {
            s +=
              '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:' +
              this.series.color +
              '">' +
              this.series.name +
              ' </p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:' +
              this.series.color +
              '">' +
              this.y +
              " " +
              this.series.userOptions.tooltip.valueSuffix +
              "</p><br/>";
          });

          return s;
        },
      },
    };

    for (let index in this.data) {
      if (this.data[index]['group'] != undefined) {
        this.setGroupData(index);
      }
    }

    this.solutions.forEach((solution, index) => {
      this.geojsonobj[solution.name] = {};
    });

    mapboxgl.accessToken =
      "pk.eyJ1IjoiZGF0YWtvcnVtLWRldmVsb3AiLCJhIjoiY2w0ODhqOXc2MDR1MjNjbzJ2YnZkdjNhYSJ9.pVPMnmkwOuF4DonlbNkkEw";

    const map = new mapboxgl.Map({
      attributionControl: false,
      container: "map",
      style: "mapbox://styles/datakorum-develop/cl4cu370w000v15tal53sxn9v",
      center: [this.location.GPS_lng, this.location.GPS_lat],
      zoom: 17,
      pitch: 60,
      bearing: 0,
      //maxZoom: 20,
      minZoom: 5,
      maxPitch: 67.5,
    });

    let geojsonobj;
    var leaverdevices = 1,
      leaverpopup = 1;
    var popup, popups, marker_type, coordinates, description, solution_type;

    map.on("load", () => {

      map.loadImage("assets/images/markers/marker_park_aval.png", (error, image) => {
        if (error) throw error;
        map.addImage("marker_park_aval", image);
      });

      map.loadImage("assets/images/markers/marker_park.png", (error, image) => {
        if (error) throw error;
        map.addImage("marker_park", image);
      });

      map.loadImage("assets/images/markers/marker_park_take.png", (error, image) => {
        if (error) throw error;
        map.addImage("marker_park_take", image);
      });

      map.loadImage(
        "assets/images/markers/marker_city_enabled.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_city_enabled", image);
        }
      );

      map.loadImage(
        "assets/images/markers/marker_city_disabled.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_city_disabled", image);
        }
      );

      map.loadImage("assets/images/markers/marker_city_alert.png", (error, image) => {
        if (error) throw error;
        map.addImage("marker_city_alert", image);
      });

      map.loadImage(
        "assets/images/markers/marker_energy_enabled.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_energy_enabled", image);
        }
      );

      map.loadImage(
        "assets/images/markers/marker_energy_disabled.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_energy_disabled", image);
        }
      );

      map.loadImage(
        "assets/images/markers/marker_energy_alert.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_energy_alert", image);
        }
      );

      map.loadImage(
        "assets/images/markers/marker_mobility_enabled.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_mobility_enabled", image);
        }
      );

      map.loadImage(
        "assets/images/markers/marker_mobility_disabled.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_mobility_disabled", image);
        }
      );

      map.loadImage(
        "assets/images/markers/marker_mobility_alert.png",
        (error, image) => {
          if (error) throw error;
          map.addImage("marker_mobility_alert", image);
        }
      );

      map.loadImage("assets/images/markers/cluster_marker.png", (error, image) => {
        if (error) throw error;
        map.addImage("cluster_marker", image);
      });

      this.solutions.forEach((solution, index) => {
        geojsonobj = {};
        var newFeature = {};
        geojsonobj["type"] = "FeatureCollection";
        geojsonobj["features"] = [];

        this.gateways.forEach((gateway) => {

          description = '';

          if (solution.id.includes(gateway.solutions_id)) {
            solution_type = gateway.solutions_name;

            if (gateway.type_name.indexOf("PULSES") != -1) {

              marker_type = "marker_" + gateway.solutions_name;

              description =
                '<div class="content-between m-0 cursor-default text-xs">' +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Name") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.name +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Id") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.id +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Group") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.group_name +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Type") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.type_name +
                "</p></div>";

              if (gateway.active == 0) {
                //0 inactivo, 1 activo
                description +=
                  '<div class="grid grid-cols-1 text-center text-gray-light gap-2 mt-2">' +
                  '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
                  '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                  "&nbsp;" +
                  this.$t("general.Profile") +
                  "" +
                  "</div>" +
                  "</div>";
                marker_type += "_disabled";
              } else if (gateway.active == 1) {

                description +=
                  '<div class="flex justify-between text-cyan-text"><p>' +
                  this.$t("general.Last sent") +
                  ' &nbsp</p><p id="popupDate_' + gateway.id + '" class="text-gray-light whitespace-nowrap"></p></div>' +
                  '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
                  '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
                  '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                  "&nbsp;" +
                  this.$t("general.Readings") +
                  "" +
                  "</div>" +
                  '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
                  '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                  "&nbsp;" +
                  this.$t("general.Profile") +
                  "" +
                  //"</div>" +
                  "</div>";
                marker_type += "_enabled";
              }

              if(gateway.solutions_id == 9){

                description +=
                  '<div class="col-span-2 rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center gap-1" id="btnValve">' +
                  this.$t("general.Valve") +
                  "" +
                  "</div>";
              }

              description += '</div>';

            } else if (gateway.type_name.indexOf("PARK") != -1) {

              if (typeof this.data["PARK"] != 'undefined' && typeof this.data["PARK"][gateway.id] != 'undefined' && this.data["PARK"][gateway.id]) {
                marker_type = "marker_park_";

                gateway.active = 1;

                let lastValues = this.data["PARK"][gateway.id];
                let date = lastValues['date'];

                let duration = this.formatTime(date, true, true, gateway.hoursOffset);
                let state = "...";

                description =
                  '<div class="content-between m-0 cursor-default text-xs"><div class="flex justify-between">';

                if (lastValues['lecture_value'] == 1) {
                  state = "Taken-spot";
                  marker_type += "take";
                  description += '<p class="text-Taken-spot text-base font-semibold items-baseline">' +
                    this.$t("general." + state) +
                    '</p><div class="grid grid-cols-2 place-items-center gap-1">';
                } else if (
                  lastValues['lecture_value'] == 0
                ) {
                  state = "Available-spot";
                  marker_type += "aval";
                  description += '<p class="text-Available-spot text-base font-semibold items-baseline">' +
                    this.$t("general." + state) +
                    '</p><div class="grid grid-cols-2 place-items-center gap-1">';
                } else {
                  marker_type = "marker_park";
                  description += '<p></p><div class="grid grid-cols-2 place-items-center gap-1">';
                }

                if (lastValues['RSSI'] <= -110) {
                  description += '<i class="DK-signal-01" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative"></i>';
                } else if (lastValues['RSSI'] <= -100) {
                  description += '<i class="DK-signal-02" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                } else if (lastValues['RSSI'] <= -85) {
                  description += '<i class="DK-signal-03" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                } else if (lastValues['RSSI'] <= -70) {
                  description += '<i class="DK-signal-04" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                } else {
                  description += '<i class="DK-signal-05" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                }

                if (
                  lastValues["Battery Level"] <= 2.9
                ) {
                  description += '<i class="DK-Batt-25 animate-pulse" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                } else if (
                  lastValues["Battery Level"] <= 3.1
                ) {
                  description += '<i class="DK-Batt-50" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                } else if (
                  lastValues["Battery Level"] <= 3.3
                ) {
                  description += '<i class="DK-Batt-75" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                } else {
                  description += '<i class="DK-Batt-100" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                }

                description += "</div></div>";

                description +=
                  '<div class="flex justify-start text-cyan-text px-0.5"><p class="text-gray-light whitespace-nowrap">' +
                  duration +
                  "</p></div>" +
                  '<div class="border-t border-gray-medium my-1"></div>' +
                  '<div class="flex justify-between text-cyan-text"><p>' +
                  this.$t("general.Name") +
                  ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                  gateway.name +
                  "</p></div>" +
                  '<div class="flex justify-between text-cyan-text"><p>' +
                  this.$t("general.Last sent") +
                  ' &nbsp</p><p id="popupDate_' + gateway.id + '" class="text-gray-light whitespace-nowrap">' +
                  date +
                  "</p></div>" +
                  '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
                  '<div class="rounded-md col-span-2 bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
                  '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                  "&nbsp;" +
                  this.$t("general.Readings") +
                  "" +
                  "</div>" +
                  "</div>";

              } else {
                marker_type = "marker_park_";

                //let lastValues = this.data["PARK"][gateway.id];
                let date = this.$t('general.No sendings yet');

                let duration = '';
                let state = "No sendings yet";

                description = '<div class="content-between m-0 cursor-default text-xs"><div class="flex justify-between">';

                marker_type = "marker_park";

                description += '<p class="text-gray-light text-base font-semibold items-baseline">' +
                  this.$t("general." + state) +
                  '</p><div class="grid grid-cols-2 place-items-center gap-1">';

                description += '<i class="DK-signal-01" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative"></i>';

                description += '<i class="DK-Batt-25 animate-pulse" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';

                description += "</div></div>";

                description +=
                  '<div class="flex justify-start text-cyan-text px-0.5"><p class="text-gray-light whitespace-nowrap">' +
                  duration +
                  "</p></div>" +
                  '<div class="border-t border-gray-medium my-1"></div>' +
                  '<div class="flex justify-between text-cyan-text"><p>' +
                  this.$t("general.Name") +
                  ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                  gateway.name +
                  "</p></div>" +
                  '<div class="flex justify-between text-cyan-text"><p>' +
                  this.$t("general.Last sent") +
                  ' &nbsp</p><p id="popupDate_' + gateway.id + '" class="text-gray-light whitespace-nowrap">' +
                  date +
                  "</p></div>" +
                  '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
                  '<div class="rounded-md col-span-2 bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
                  '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                  "&nbsp;" +
                  this.$t("general.Readings") +
                  "" +
                  "</div>" +
                  "</div>";
              }
            } else if (gateway.type_name.indexOf("MODBUS") != -1 || gateway.MODBUS_device_type != "") {
              marker_type = "marker_" + gateway.solutions_name;

              description =
                '<div class="content-between m-0 cursor-default text-xs">' +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Name") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.name +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Id") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.id +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Group") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.group_name +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Type") +
                ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                gateway.type_name +
                "</p></div>";

              /*if (gateway.active == 0) {
                description +=
                  '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
                    '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                    "&nbsp;" +
                    this.$t("general.Readings") +
                    "" +
                    "</div>" +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
                    '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                    "&nbsp;" +
                    this.$t("general.Profile") +
                    "" +
                  "</div>";
                marker_type += "_disabled";
              } else if (gateway.active == 1) { */

                description +=
                  '<div class="flex justify-between text-cyan-text"><p>' +
                  this.$t("general.Last sent") +
                  ' &nbsp</p><p id="popupDate_' + gateway.id + '" class="text-gray-light whitespace-nowrap"></p></div>' +
                  '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
                    '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                    "&nbsp;" +
                    this.$t("general.Readings") +
                    "" +
                    "</div>" +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
                    '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                    "&nbsp;" +
                    this.$t("general.Profile") +
                    "" +
                  "</div>";

                if(gateway.MODBUS_device_type == 6 || gateway.MODBUS_device_type == 7){

                  description +=
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnState">' +
                    this.$t("general.State") +
                    "" +
                    "</div>" +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnParameters">' +
                    this.$t("general.Parameters") +
                    "" +
                    "</div>";
                }

                if(gateway.MODBUS_device_type == 23){

                  description +=
                    '<div class="col-span-2 rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center gap-2" id="btnRTDashboard">' +
                      //'<i class="fa-solid fa-sliders" style="font-size:0.875rem; color:#FFFAFA;"></i>' +
                      this.$t("general.Dashboard") +
                      "" +
                    "</div>";
                }

                if(gateway.MODBUS_device_type == 34){

                  description +=
                    '<div class="col-span-2 rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center gap-2" id="btnOptidrive">' +
                      //'<i class="fa-solid fa-sliders" style="font-size:0.875rem; color:#FFFAFA;"></i>' +
                      this.$t("general.Dashboard") +
                      "" +
                    "</div>";
                }

                if(gateway.MODBUS_device_type == 31){

                  description +=
                    '<div class="col-span-2 rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center gap-2" id="btnWilo">' +
                      this.$t("general.Dashboard") +
                      "" +
                    "</div>";
                }

                if(gateway.solutions_id == 8){

                  description +=
                    '<div class="col-span-2 rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center gap-1" id="btnMeteo">' +
                    this.$t("general.Meteo") +
                    "" +
                    "</div>";
                }

                description += '</div>';
                marker_type += "_enabled";

            } else if (gateway.type_name.indexOf("MB") != -1 || gateway.type_name.indexOf('WM') !== -1) {
                marker_type = "marker_" + gateway.solutions_name;

                description =
                    '<div class="content-between m-0 cursor-default text-xs">' +
                    '<div class="flex justify-between text-cyan-text"><p>' +
                    this.$t("general.Name") +
                    ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                    gateway.name +
                    "</p></div>" +
                    '<div class="flex justify-between text-cyan-text"><p>' +
                    this.$t("general.Id") +
                    ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                    gateway.id +
                    "</p></div>" +
                    '<div class="flex justify-between text-cyan-text"><p>' +
                    this.$t("general.Group") +
                    ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                    gateway.group_name +
                    "</p></div>" +
                    '<div class="flex justify-between text-cyan-text"><p>' +
                    this.$t("general.Type") +
                    ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                    gateway.type_name +
                    "</p></div>";

                if (gateway.active == 0) {
                    description +=
                    '<div class="grid grid-cols-1 text-center text-gray-light gap-2 mt-2">' +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
                    '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                    "&nbsp;" +
                    this.$t("general.Profile") +
                    "" +
                    "</div>" +
                    "</div>";
                    marker_type += "_disabled";
                } else if (gateway.active == 1) {

                    description +=
                    '<div class="flex justify-between text-cyan-text"><p>' +
                    this.$t("general.Last sent") +
                    ' &nbsp</p><p id="popupDate_' + gateway.id + '" class="text-gray-light whitespace-nowrap"></p></div>' +
                    '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
                    '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                    "&nbsp;" +
                    this.$t("general.Readings") +
                    "" +
                    "</div>" +
                    '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
                    '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                    "&nbsp;" +
                    this.$t("general.Profile") +
                    "" +
                    "</div>";
                }

                if(gateway.solutions_id == 11){

                  description +=
                    '<div class="col-span-2 rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center gap-1" id="btnUllastres">' +
                    this.$t("general.Dashboard") +
                    "" +
                    "</div>";
                }

                description += '</div>';
                marker_type += "_enabled";
            }

            description += "</div>";

            newFeature = {
              type: "Feature",
              properties: {
                name: gateway.name,
                type_name: gateway.type_name,
                group_name: gateway.group_name,
                id: gateway.id,
                idtype: gateway.idtype,
                marker_type: marker_type,
                status: gateway.active,
                solution_type: solution_type,
                description: description,
                insidescreen: "",
              },
              geometry: {
                type: "Point",
                coordinates: [gateway.GPS_lng, gateway.GPS_lat],
              },
            };

            for (var index = 0; index < this.alerts_log.length; index++) {
              if (
                this.alerts_log[index].read == 0 &&
                newFeature.properties.id == this.alerts_log[index].iddevice
              ) {
                newFeature.properties.marker_type =
                  "marker_" + gateway.solutions_name + "_alert";
              }
            }

            geojsonobj["features"].push(newFeature);
            this.mqtt_connection(this.user.idgroup_selected, gateway.id);

          }
        });

        this.geojsonobj[solution_type] = geojsonobj;

        // Add data to map
        map.addSource(solution.name, {
          type: "geojson",
          data: geojsonobj,
          cluster: true,
          clusterMaxZoom: 20,
          clusterRadius: this.screen_width >= 768 ? 20 : 40,
        });

        // Layer individual device
        let def_marker
        if(solution.name.toLowerCase() == 'park'){
          def_marker = 'marker_park';
        }else{
          def_marker = 'cluster_marker';
        }

        map.addLayer({
          id: solution.name,
          type: "symbol",
          source: solution.name,
          filter: ["!", ["has", "point_count"]],
          layout: {
            "icon-image": [
              "match",
              ["get", "marker_type"],

              "marker_park_take",
              "marker_park_take",
              "marker_park_aval",
              "marker_park_aval",
              "marker_park",
              "marker_park",

              "marker_city_enabled",
              "marker_city_enabled",
              "marker_city_disabled",
              "marker_city_disabled",
              "marker_city_alert",
              "marker_city_alert",

              "marker_energy_enabled",
              "marker_energy_enabled",
              "marker_energy_disabled",
              "marker_energy_disabled",
              "marker_energy_alert",
              "marker_energy_alert",

              "marker_mobility_enabled",
              "marker_mobility_enabled",
              "marker_mobility_disabled",
              "marker_mobility_disabled",
              "marker_mobility_alert",
              "marker_mobility_alert",

              "clusterLang",
              "cluster_marker",

              def_marker, // default
            ],
            "icon-anchor": "bottom",
            "icon-size": 0.9,
            visibility: "visible",
          },
        });

        // CLICK THE DEVICE
        map.on("click", solution.name, (e) => {
          this.week_day = new Date().getDay();

          coordinates = this.coordinatesFromMarker(e);
          this.show(
            e,
            e.features[0].properties.id,
            e.features[0].properties.status,
            map
          );

          this.id_open = e.features[0].properties.id;

          if (this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0") {
            //cuando esté abierto
            const description = e.features[0].properties.description;

            popup = new mapboxgl.Popup()
              .setLngLat(coordinates)
              .setHTML(description);
          }

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (popups.length) {
            popups[0].remove();
          }

          popup.addTo(map);
        });

        // CLICK OUT OF THE DEVICE
        map.on("click", (event) => {

          if (this.user.admin == 0 && event.originalEvent.ctrlKey) {
            let coordenates =
            {
              lat: event.lngLat.lat,
              lng: event.lngLat.lng
            }

            navigator.clipboard.writeText(coordenates.lat + ' ' + coordenates.lng)
              .then(() => {
              })
              .catch((error) => {
                window.emitter.emit('changeLoaderStatus', false, 'orbit')

                if(error.response && error.response.status && error.response.status == 429){
                  window.alert( this.$t('text.Api attempts'));
                }else{
                  //console.error('Error al copiar el texto: ', error);
                  return "Failed!", error;
                }
              });
          }

          let status;

          this.solutions.forEach((solution, index) => {
            this.geojsonobj[solution.name].features.forEach((feature) => {
              if (this.id_open == feature.properties.id) {
                status = feature.properties.status;
              }
            });
          });

          if (
            this.screen_width >= 768 &&
            document.getElementById("sidemenu").clientWidth != "0" &&
            (this.charts[0] != undefined || status == 0)
          ) {
            this.hide();
          }
          this.showlist = false;
        });

        // HOVER THE DEVICE
        map.on("mouseenter", solution.name, (e) => {
          leaverdevices = 0;

          const description = e.features[0].properties.description;

          if (this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth == "0") {
            this.id_open = e.features[0].properties.id;
          }

          coordinates = this.coordinatesFromMarker(e);

          popup = new mapboxgl.Popup()
            .setLngLat(coordinates)
            .setHTML(description);

          if (!(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")) {
            // Obtener el ultimo dato
            if (e.features[0].properties.status == 1)
              this.getLastData(e.features[0].properties.id).then((response) => {

                if (response.type_name == 'MODBUS') {

                  const date = response.dates[0];
                  if (
                    document.getElementsByClassName("mapboxgl-popup").length &&
                    document.getElementById("popupDate_" + e.features[0].properties.id)
                  ) {
                    document.getElementById("popupDate_" + e.features[0].properties.id).innerHTML = date;;
                  }

                } else if (response.type_name == 'PARK') {

                  /*if (this.data['PARK'][Object.keys(response['PARK'])[0]].registers[Object.keys(this.data['PARK'][Object.keys(response['PARK'])[0]].registers)[0]].date != Object.keys(response['PARK'][Object.keys(response['PARK'])[0]].registers)[0]) {
                    this.data['PARK'][Object.keys(response['PARK'])[0]].registers[Object.keys(response['PARK'][Object.keys(response['PARK'])[0]].registers)[0]] = response['PARK'][Object.keys(response['PARK'])[0]].registers[0];
                    this.data['PARK'][Object.keys(response['PARK'])[0]].charts['occupancy'].push(response['PARK'][Object.keys(response['PARK'])[0]].charts['occupancy'][0])
                    this.data['PARK'][Object.keys(response['PARK'])[0]].charts['rotations'][(new Date(response['PARK'][Object.keys(response['PARK'])[0]].charts['occupancy'][0][0])).getUTCHours().toString()] += 1
                  }*/



                } else if(response.type_name == 'PULSES') {
                    if(
                      document.getElementsByClassName("mapboxgl-popup").length &&
                      document.getElementById("popupDate_" + e.features[0].properties.id)
                      ){

                        document.getElementById("popupDate_" + e.features[0].properties.id).innerHTML = response.dates[0];
                    }


                } else if (response.type_name == null) {
                  if (
                    document.getElementsByClassName("mapboxgl-popup").length &&
                    document.getElementById("popupDate_" + e.features[0].properties.id)
                  ) {
                    document.getElementById("popupDate_" + e.features[0].properties.id).innerHTML = 'UNDEFINED';
                  }
                }

              });
          }

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (
            popups.length &&
            !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")
          ) {
            popups[0].remove();
          }

          if (!(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")) {
            popup.addTo(map);
          }

          map.getCanvas().style.cursor = "pointer";
        });

        // HOVER THE DEVICE
        $(document).on("mouseenter", ".mapboxgl-popup", () => {
          leaverpopup = 0;

          // Para los botones del POPUP
          if (document.getElementById("btnProfile") != null)
            document
              .getElementById("btnProfile")
              .addEventListener("click", () => {
                document.getElementById('LinkProfile_' + this.id_open).click();
          });

          if (document.getElementById("btnReadings") != null)
            document
              .getElementById("btnReadings")
              .addEventListener("click", () => {
                document.getElementById('LinkReadings_' + this.id_open).click();
          });

          if (document.getElementById("btnState") != null)
            document
              .getElementById("btnState")
              .addEventListener("click", () => {
                document.getElementById('LinkState_' + this.id_open).click();
          });

          if (document.getElementById("btnMeteo") != null)
            document
              .getElementById("btnMeteo")
              .addEventListener("click", () => {
                document.getElementById('LinkMeteo_' + this.id_open).click();
          });

          if (document.getElementById("btnValve") != null)
            document
              .getElementById("btnValve")
              .addEventListener("click", () => {
                document.getElementById('LinkValve_' + this.id_open).click();
          });

          if (document.getElementById("btnRTDashboard") != null)
            document
              .getElementById("btnRTDashboard")
              .addEventListener("click", () => {
                document.getElementById('LinkRTDashboard_' + this.id_open).click();
          });

          if (document.getElementById("btnOptidrive") != null)
            document
              .getElementById("btnOptidrive")
              .addEventListener("click", () => {
                document.getElementById('LinkOptidrive_' + this.id_open).click();
          });

          if (document.getElementById("btnWilo") != null)
            document
              .getElementById("btnWilo")
              .addEventListener("click", () => {
                document.getElementById('LinkWilo_' + this.id_open).click();
          });

          if (document.getElementById("btnParameters") != null)
            document
              .getElementById("btnParameters")
              .addEventListener("click", () => {
                document.getElementById('LinkParameters_' + this.id_open).click();
          });

          if (document.getElementById("btnUllastres") != null)
            document
              .getElementById("btnUllastres")
              .addEventListener("click", () => {
                document.getElementById('LinkUllastres_' + this.id_open).click();
          });

        });

        // UNHOVER THE DEVICE
        map.on("mouseleave", solution.name, (e) => {
          popups = document.getElementsByClassName("mapboxgl-popup");

          setTimeout(() => {
            leaverdevices = 1;
            if (
              !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0") &&
              leaverdevices * leaverpopup == 1 &&
              popups[0]
            ) {
              popups[0].remove();
            }
          }, 100);

          map.getCanvas().style.cursor = "";
        });

        // Layer cluster
        map.addLayer({
          id: "cluster_" + solution.name,
          type: "symbol",
          source: solution.name,
          filter: ["has", "point_count"],
          layout: {
            "icon-image": "cluster_marker",
            "icon-allow-overlap": true,
            "icon-anchor": "bottom",
            "icon-size": 1,
            "text-field": "{point_count_abbreviated}",
            "text-font": ["Roboto Bold"],
            "text-size": 11,
            "text-allow-overlap": true,
            "text-anchor": "bottom",
            "text-offset": [0, -1],
          },
          paint: {
            "text-color": "#FFFAFA",
          },
        });

        // CLICK IN THE CLUSTER
        map.on("click", "cluster_" + solution.name, (e) => {
          const features = map.queryRenderedFeatures(e.point, {
            layers: ["cluster_" + solution.name],
          });

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (popups.length) {
            popups[0].remove();
          }

          const clusterId = features[0].properties.cluster_id;
          map
            .getSource(solution.name)
            .getClusterExpansionZoom(clusterId, (err, zoom) => {
              if (err) {
                return;
              }
              map.flyTo({
                center: features[0].geometry.coordinates,
                zoom: zoom + 3,
              });
              this.showlist = true;
            });
        });

        // HOVER THE CLUSTER
        map.on("mouseenter", "cluster_" + solution.name, (e) => {
          coordinates = this.coordinatesFromMarker(e);

          popup = new mapboxgl.Popup().setLngLat(coordinates);

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (
            popups.length &&
            !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")
          ) {
            popups[0].remove();
          }

          if (!(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")) {
            popup.addTo(map);
          }

          map.getCanvas().style.cursor = "pointer";
        });

        // UNHOVER THE CLUSTER
        map.on("mouseleave", "cluster_" + solution.name, (e) => {
          map.getCanvas().style.cursor = "";
        });
      });

      // UNHOVER THE POPUP
      $(document).on("mouseleave", ".mapboxgl-popup", () => {
        popups = document.getElementsByClassName("mapboxgl-popup");

        setTimeout(() => {
          leaverpopup = 1;
          if (
            !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0") &&
            leaverdevices * leaverpopup == 1 &&
            popups[0]
          ) {
            popups[0].remove();
          }
        }, 100);
      });

      // RENDERS MAP
      var rendercounter = 0;

      map.on("render", () => {
        var temp1 = rendercounter + 1;

        setTimeout(() => {
          if (temp1 == rendercounter) {
            this.isOnScreen(map);
            this.searchfilter();
            this.directAccessList(map);
          }
        }, 12);

        rendercounter++;
      });

      var bounds = new mapboxgl.LngLatBounds();

      this.MapFitBounds(500, map, bounds);
    });

    // After the last frame rendered before the map enters an "idle" state.
    map.on("idle", () => {
      // Enumerate ids of the layers.
      let toggleableLayerIds = new Array();

      this.solutions.forEach((solution, index) => {
        toggleableLayerIds.push(solution.name);
      });

      // Set up the corresponding toggle button for each layer.
      for (const id of toggleableLayerIds) {
        // Skip layers that already have a button set up.

        if (document.getElementById(id)) {
          continue;
        }

        // Create a link.
        const link = document.createElement("a");
        link.id = id;
        link.href = "#";
        //link.textContent = id;
        link.innerHTML =
          '<img class="h-4" src="/assets/images/markers/marker_' +
          id +
          '.png" /><a>' +
          id.charAt(0).toUpperCase() + id.slice(1).toLowerCase() +
          "</a>";
        link.className = "active inline-flex gap-1 justify-start items-center";

        // Show or hide layer when the toggle is clicked.
        link.onclick = (e) => {
          const clickedLayer = (link.textContent).toLowerCase();
          e.preventDefault();
          e.stopPropagation();

          var visibility = map.getLayoutProperty(clickedLayer, "visibility");

          // Toggle layer visibility by changing the layout object's visibility property.
          if (visibility === "visible") {
            map.setLayoutProperty(clickedLayer, "visibility", "none");
            map.setLayoutProperty('cluster_' + clickedLayer, "visibility", "none");
            link.className =
              "inline-flex gap-1 justify-start items-center text-gray-medium";
          } else {
            link.className =
              "active inline-flex gap-1 justify-start items-center";
            map.setLayoutProperty(clickedLayer, "visibility", "visible");
            map.setLayoutProperty('cluster_' + clickedLayer, "visibility", "visible");
          }

        };

        const layers = document.getElementById("legend");
        layers.appendChild(link);
      }
    });

    this.map = map;

    /*setTimeout(() => {

      let title = "Hola Mundo";
      let message = "Hola Mundo";

      this.Notify(title, message, 2)

    }, 3000);*/


  },

  methods: {

    mqtt_connection(idgroup, iddevice) {

      let idsolution;
      this.solutions.forEach((solution, index) => {
        this.gateways.forEach((gateway) => {
          if (gateway.id == iddevice && solution.id == gateway.solutions_id) {
            idsolution = solution.id
          }
        })
      })

      // mqtt_topic = "/PUSHER/@P@" + this.user.idplatform + "/G@" + idgroup + "/@D@" + iddevice;

      var mqtt_topic = "/PUSHER/@P@" + this.user.idplatform + "/G@" + idgroup;

      if (typeof idgroup != 'undefined'){
        mqtt_topic += "/@D@" + iddevice
      }

      const mqtt = require('paho-mqtt');

      const mqtt_host = 'mqtt.datakorum.app';
      const mqtt_port = 8000;
      const mqtt_path = '';

      var userName = this.user.credentials.username;
      var password = this.user.credentials.password;

      var clientId = "APPclientid_" + parseInt(Math.random() * 100, 10);

      const client = new mqtt.Client(mqtt_host, mqtt_port, mqtt_path, clientId);

      client.connect({

        useSSL: true,
        mqttVersion: 4,
        userName: userName,
        password: password,

        onSuccess: () => {
          // función de éxito de conexión

          client.subscribe(mqtt_topic, {

            onSuccess: () => {
              // función de éxito de suscripción
              console.log('éxito de suscripción ' + iddevice)
            },

            onFailure: () => {
              // función de fallo de suscripción
              console.error('fallo de suscripción ' + iddevice)
            }

          });
        },

        onFailure: () => {
          // función de fallo de conexión
          console.error('fallo de conexión ' + iddevice)
        }

      });

      client.onConnectionLost = () => {
        // función que se ejecuta cuando perdemos la conexión
        console.error('perdida de conexión ' + iddevice)

        setTimeout(() => {
          this.mqtt_connection(iddevice)
        }, 15000);

      };

      client.onMessageArrived = (message) => {
        // función que se ejecuta cuando llega un mensaje al topic suscrito

        //this.Notify('New sent', 'We recieved a new ', 2)

        var map = this.map;

        var data = JSON.parse(JSON.parse(message.payloadString).frame_data);

        if(idsolution == 6){// park

          for (let index in this.data) {

            if (typeof this.data[index][iddevice] !== undefined) {

              if (index == 'PARK') {

                let temp_obj = {};
                temp_obj.type_name = index;
                temp_obj[index] = {};
                temp_obj[index][iddevice] = null;

                let state = parseInt(data.lecture_value);

                let date = this.formatDate(new Date());

                this.data[index][iddevice].registers.unshift(data)

                this.data[index][iddevice].registers[0].date = date;
                this.data[index][iddevice].registers[0].epoch = (new Date(date)).getTime();

                if (this.data[index][iddevice].charts.occupancy[this.data[index][iddevice].charts.occupancy.length - 1][1] == this.data[index][iddevice].registers[0].lecture_value) {
                  this.data[index][iddevice].charts.occupancy.pop();
                }

                this.data[index][iddevice].charts.occupancy.push([(new Date(date)).getTime(), state]);

                if (this.data[index]['group'].charts.occupancy[this.data[index]['group'].charts.occupancy.length - 1][1] == this.data[index]['group'].charts.occupancy[this.data[index]['group'].charts.occupancy.length - 2][1]) {
                  this.data[index]['group'].charts.occupancy.pop();
                }

                this.data[index]['group'].charts.occupancy.push([
                  (new Date(date)).getTime(),
                  state == 1 ? this.data[index]['group'].charts.occupancy[this.data[index]['group'].charts.occupancy.length - 1][1] + 1 : this.data[index]['group'].charts.occupancy[this.data[index]['group'].charts.occupancy.length - 1][1] - 1
                ]);

                this.data[index][iddevice].charts.rotations[new Date().getHours()] += 1;
                this.data[index][iddevice]['total_rotation_times'] += 1;
                this.data[index]['group'].charts.rotations[new Date().getHours()] += 1;
                this.data[index]['group']['total_rotation_times'] += 1;

                this.setDeviceData(temp_obj);
                this.setGroupData(index);

                var layerId = index.toLowerCase();
                var RSSI = data['RSSI'];
                var BatteryLevel = data['Battery Level'];
                var layer = map.getLayer(layerId);

                if (layer) {

                  let geojsonobj = this.geojsonobj[layerId]

                  geojsonobj.features.forEach((feature, index) => {

                    if (feature.properties.id == iddevice) {
                      geojsonobj.features[index].properties.marker_type = state == 1 ? "marker_park_take" : "marker_park_aval";
                      geojsonobj.features[index].properties.status = state;


                      let duration = this.formatTime(date, true, true);

                      let description = '<div class="content-between m-0 cursor-default text-xs"><div class="flex justify-between">';

                      if (state == 1) {
                        description +=       '<p class="text-Taken-spot text-base font-semibold items-baseline">' +
                          this.$t("general.Taken-spot") +
                          '</p><div class="grid grid-cols-2 place-items-center gap-1">';
                      } else if (state == 0) {
                        description +=       '<p class="text-Available-spot text-base font-semibold items-baseline">' +
                          this.$t("general.Available-spot") +
                          '</p><div class="grid grid-cols-2 place-items-center gap-1">';
                      } else {
                        description +=       '<p></p><div class="grid grid-cols-2 place-items-center gap-1">';
                      }

                      if (RSSI <= -110) {
                        description +=       '<i class="DK-signal-01" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative"></i>';
                      } else if (RSSI <= -100) {
                        description +=       '<i class="DK-signal-02" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                      } else if (RSSI <= -85) {
                        description +=       '<i class="DK-signal-03" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                      } else if (RSSI <= -70) {
                        description +=       '<i class="DK-signal-04" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                      } else {
                        description +=       '<i class="DK-signal-05" style="font-size:0.8rem; margin-right:0rem; color:#d1d1d1; margin-left:0rem; position:relative;"></i>';
                      }

                      if (
                        BatteryLevel <= 2.9
                      ) {
                        description +=       '<i class="DK-Batt-25 animate-pulse" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                      } else if (
                        BatteryLevel <= 3.1
                      ) {
                        description +=       '<i class="DK-Batt-50" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                      } else if (
                        BatteryLevel <= 3.3
                      ) {
                        description +=       '<i class="DK-Batt-75" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                      } else {
                        description +=       '<i class="DK-Batt-100" style="font-size:1.2rem; color:#d1d1d1; margin-right:0rem;  margin-left:0rem; position:relative"></i>';
                      }

                      description +=     '</div></div>' +
                        '<div class="flex justify-start text-cyan-text px-0.5"><p class="text-gray-light whitespace-nowrap">' +
                        duration +
                        "</p></div>" +
                        '<div class="border-t border-gray-medium my-1"></div>' +
                        '<div class="flex justify-between text-cyan-text"><p>' +
                        this.$t("general.Name") +
                        ' &nbsp</p><p class="text-gray-light whitespace-nowrap">' +
                        geojsonobj.features[index].properties.name +
                        "</p></div>" +
                        '<div class="flex justify-between text-cyan-text"><p>' +
                        this.$t("general.Last sent") +
                        ' &nbsp</p><p id="popupDate_' + geojsonobj.features[index].properties.id + '" class="text-gray-light whitespace-nowrap">' +
                        date +
                        "</p></div>" +
                        '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
                        '<div class="rounded-md col-span-2 bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
                        '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                        "&nbsp;" +
                        this.$t("general.Readings") +
                        "" +
                        "</div>" +
                        '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
                        '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
                        "&nbsp;" +
                        this.$t("general.Profile") +
                        "" +
                        "</div>" +
                        "</div>" +
                        "</div>";

                      geojsonobj.features[index].properties.description = description;
                    }
                  })

                  map.getSource(layerId).setData(geojsonobj);

                  let popups = document.getElementsByClassName("mapboxgl-popup");
                  if (popups[0]) {
                    popups[0].remove();
                  }
                }
              }
            }
          }
        }
      };
    },

    setGroupData(index) {

      if(this.screen_width >= 768){

        const Highcharts = require("highcharts");
        let days = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];

        let default_highcharts_charts = new Array();

        let Chart = this.Chart;
        let $t = this.$t;

        //CHART BOOL

        this.default_charts[0] = {
          name: this.$t("general.Todaysoccupancy"),
        }

        Chart.yAxis.tickPositions = [0, this.data[index]['group']['charts']['occupancy'][this.data[index]['group']['charts']['occupancy'].length - 1][1], this.data[index]['group']['num_devices']];
        Chart.yAxis.opposite = true;
        Chart.yAxis.gridLineColor = 'rgba(0,0,0,0)'
        Chart.yAxis.labels = {
          style: {
            color: "rgba(200, 200, 200, 0.9)",
            opacity: 0.7,
          },
          formatter: function () {
            if (this.value !== Chart.yAxis.tickPositions[0] && this.value !== Chart.yAxis.tickPositions[2]) {
              return this.value;
            }
          }
        }

        Chart.tooltip = {
          shared: true,
          split: false,
          backgroundColor: "rgba(7,82,103,0.95)",
          borderRadius: 12,
          borderWidth: 1,
          borderColor: "rgba(250, 250, 250, 0.65)",
          padding: 6,

          formatter: function () {
            const date = new Date(this.x);

            const year = date.getFullYear();
            const month = ('0' + (date.getMonth() + 1)).slice(-2);
            const day = ('0' + date.getDate()).slice(-2);
            const hours = ('0' + date.getHours()).slice(-2);
            const minutes = ('0' + date.getMinutes()).slice(-2);
            const formattedDateTime = `${year}-${month}-${day} ${hours}:${minutes}`;

            var s = '';

            $.each(this.points, function () {

                if (this.y == 1) {

                    s +=
                        '<p style="font-size:16px; line-height: 16px; font-weight: 400; color:' + this.color + '"> ' +
                        this.series.userOptions.tooltip.valueSuffix +
                        "</p><br/>";
                }
            });

            s +=
                "<b>" +
                '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
                $t('general.Since') + ' ' +
                formattedDateTime +
                "</p><br/>";

            return s;
          },
        };

        Chart.xAxis = {
          startOnTick: false,
          minPadding: 0,
          lineColor: "#647F8C",
          visible: true,
          labels: {
            formatter: function () {
              return Highcharts.dateFormat('%kh', this.value); // convert UTC to local time
            },
            style: {
              color: "rgba(200, 200, 200, 0.9)",
              opacity: 0.7,
            },
          },
          type: "datetime",
        };

        Chart.series = [{
          name: this.default_charts[0].name,
          data: this.data[index]['group']['charts']['occupancy'],
          type: 'area',
          lineWidth: 1.5,
          zIndex: 1,
          step: 'left',
          marker: {
            enabled: false
          },
          tooltip: {
            valueSuffix: '/' + Chart.yAxis.tickPositions[2]
          },
          color: {
            linearGradient: {
              x1: 0,
              y1: 0,
              x2: 0,
              y2: 1
            },
            stops: [
              [0, "rgba(5,255,243,0.8)"],
              [1, "rgba(5,255,243,0.1)"]
            ]
          }
        }];


        default_highcharts_charts.push(Highcharts.chart('default_chart_0', Chart));

        //CHART Parkingspacerotation

        this.default_charts[1] = {
          name:
            this.$t("general.Parkingspacerotation"),
        };

        var dates = Array.from(Array(24).keys()).map(num => parseInt(num));

        Chart = {
          chart: {
            backgroundColor: "transparent",
            spacingTop: 15,
          },
          accessibility: {
            enabled: false,
          },
          title: {
            text: "",
          },
          xAxis: {
            tickPositions: [4, 9, 14, 19],
            categories: dates,
            lineColor: "#647F8C",
            visible: true,
            labels: {
              style: {
                color: "rgba(200, 200, 200, 0.9)",
                opacity: 0.7,
              },
            },
            type: "category",
            plotLines: [{
              value: new Date().getHours(),
              color: 'rgba(200, 200, 200, 0.6)',
              width: 1,
              dashStyle: 'Solid',
            }],
          },
          yAxis: {
            endOnTick: false,
            visible: false,
          },
          exporting: {
            enabled: false,
          },
          credits: {
            enabled: false,
          },
          legend: {
            enabled: false,
          },
          plotOptions: {
            column: {
              borderWidth: 0,
              borderRadius: 3,
              pointWidth: 5,
              pointPadding: 0.05,
              groupPadding: 0.1,
            },
          },
          series: [
            {
              name: '',
              type: "areaspline",
              data: this.data[index]['group']['charts']['avg_rotations'][this.week_day % 7],
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: "",
              },
              states: {
                hover: {
                  enabled: false
                }
              },
              color: {
                linearGradient: {
                  x1: 0,
                  y1: 0,
                  x2: 0,
                  y2: 1
                },
                stops: [
                  [0, "rgba(7,232,157,0.3)"],
                  [1, "rgba(7,232,157,0.1)"]
                ]
              }
            }, {
              name: this.default_charts[1].name,
              type: "column",
              data: this.data[index]['group']['charts']['rotations'],
              color: "#05fff3",
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: "",
              },
            }
          ],
          tooltip: {
            enabled: true,
            valueSuffix: "",
            shared: true,
            split: false,
            backgroundColor: "rgba(7,82,103,0.95)",
            borderRadius: 12,
            borderWidth: 1,
            borderColor: "rgba(250, 250, 250, 0.65)",
            padding: 6,

            formatter: function () {
              var s =
                '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
                this.x + 'h ' + $t("general.To") + ' ' + (parseInt(this.x) + 1) % 24 + 'h' +
                "</p><br/>";

              function times_func(y) {
                if (y != 1) return "general.Rotations";
                else return "general.Rotation";
              }

              $.each(this.points, function () {
                if (this.series.name != '') {
                  s +=
                    '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:#05fff3">' +
                    this.y +
                    " " +
                    $t(times_func(this.y)) +
                    this.series.userOptions.tooltip.valueSuffix +
                    "</p><br/>";
                }
              });

              return s;
            },
          },
        };

        default_highcharts_charts.push(Highcharts.chart("default_chart_1", Chart));

        var content = '';
        let total_rotation_times = this.data[index]['group']['total_rotation_times'];

        content +=
          '<div class="px-0.5 pb-3 text-center flex justify-center items-baseline">' +
          '<i class="fa-solid fa-car" style="font-size:0.75rem; color:#05fff3;"></i>' +
          '<p class="text-xs text-gray-light">&nbsp;&nbsp;' + this.$t("text.Rotations of the day") + ': ' + total_rotation_times + '</p></div>';

        document.getElementById("default_chart_text_1").innerHTML = content;

        //CHART PEAKHOURS

        this.default_charts[2] = {
          name:
            this.$t("general.PeakHours") + ' (' + this.$t("general." + days[this.week_day % 7] + "_full") + ')',
        };

        Chart = {
          chart: {
            backgroundColor: "transparent",
            spacingTop: 15,
          },
          accessibility: {
            enabled: false,
          },
          title: {
            text: "",
          },
          xAxis: {
            tickPositions: [4, 9, 14, 19],
            categories: dates,
            lineColor: "#647F8C",
            visible: true,
            labels: {
              style: {
                color: "rgba(200, 200, 200, 0.9)",
                opacity: 0.7,
              },
            },
            type: "category",
            plotLines: [{
              value: new Date().getHours(),
              color: 'rgba(200, 200, 200, 0.6)',
              width: 1,
              dashStyle: 'Solid',
            }],
          },
          yAxis: {
            endOnTick: true,
            visible: false,
            tickPositions: [0, 100],
            labels: {
              enabled: true,
            },
            showLastLabel: true,
          },
          exporting: {
            enabled: false,
          },
          credits: {
            enabled: false,
          },
          legend: {
            enabled: false,
          },
          plotOptions: {
            column: {
              borderWidth: 0,
              borderRadius: 3,
              pointWidth: 5,
              pointPadding: 0.05,
              groupPadding: 0.1,
            },
          },
          series: [
            {
              name: this.default_charts[2].name,
              type: "column",
              data: this.data[index]['group']['charts']['peakhours'][this.week_day % 7],
              zones: [
                {
                  value: 80,
                  color: "#05fff3",
                },
                {
                  value: 95,
                  color: "#05fff3",
                },
                {
                  color: "#05fff3",
                },
              ],
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: "",
              },
            },
          ],
          tooltip: {
            enabled: false,
            valueSuffix: "",
            shared: true,
            split: false,
            backgroundColor: "rgba(7,82,103,0.95)",
            borderRadius: 12,
            borderWidth: 1,
            borderColor: "rgba(250, 250, 250, 0.65)",
            padding: 6,

            formatter: function () {
              var s =
                "<b>" +
                '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
                this.x +
                "</p><br/>";
              $.each(this.points, function () {
                s +=
                  '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:#05fff3">' +
                  this.series.name +
                  ':</p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:#05fff3">' +
                  this.y +
                  "min" +
                  this.series.userOptions.tooltip.valueSuffix +
                  "</p><br/>";
              });

              return s;
            },
          },
        };

        default_highcharts_charts.push(Highcharts.chart("default_chart_2", Chart));

        let duration = this.formatTime(this.data[index]['group']['avg_parking_time'][this.week_day % 7], false, false);

        content =
          '<div class="px-0.5 pb-3 text-center flex justify-center items-baseline">' +
          '<i class="fa-regular fa-clock" style="font-size:0.75rem; color:#05fff3;"></i>' +
          '<p class="text-xs text-gray-light">&nbsp;&nbsp;' + this.$t("text.Average parking time") + ': ' +
          duration + '</p></div>';

        document.getElementById("default_chart_text_2").innerHTML = content;

        setTimeout(() => {
          for (let i_chart = 0; i_chart < default_highcharts_charts.length; i_chart++) {
            if (default_highcharts_charts[i_chart] != undefined) {
              default_highcharts_charts[i_chart].reflow();
            }
          }
        }, 0);
      }
    },

    directAccessList(map) {
      let device_list_item = document.querySelectorAll(".device_list_item");

      for (var marker = 0; marker < device_list_item.length; marker++) {
        device_list_item[marker].addEventListener("click", (event) => {
          let popup, popups;

          var targetElement = event.target.id
            .replace(/list_device_/g, "")
            .split("_");

          const description =
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .properties.description;
          const temp_id =
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .properties.id;
          const temp_status =
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .properties.status;
          this.id_open = temp_id;

          var aux_e = {};
          aux_e["lngLat"] = [];
          aux_e.lngLat = {
            lng: parseFloat(
              this.geojsonobj[targetElement[0]].features[targetElement[1]]
                .geometry.coordinates[0]
            ),
            lat: parseFloat(
              this.geojsonobj[targetElement[0]].features[targetElement[1]]
                .geometry.coordinates[1]
            ),
          };

          this.geojsonobj[targetElement[0]].features.forEach(
            (feature, index) => {
              if (feature.properties.id == temp_id) {
                marker = index;
              }
            }
          );

          this.show(aux_e, temp_id, temp_status, map);

          let coordinates = [
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .geometry.coordinates[0],
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .geometry.coordinates[1],
          ];

          popup = new mapboxgl.Popup()
            .setLngLat(coordinates)
            .setHTML(description);

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (popups.length) {
            popups[0].remove();
          }

          popup.addTo(map);
        });
      }
    },

    isOnScreen(map) {
      if (document.getElementById("device_list")){
        document.getElementById("device_list").innerHTML = "";

        this.solutions.forEach((solution, index) => {
          this.geojsonobj[solution.name].features.forEach((feature) => {
            if (
              map.getBounds()._sw.lng <= feature.geometry.coordinates[0] &&
              feature.geometry.coordinates[0] <= map.getBounds()._ne.lng &&
              map.getBounds()._sw.lat <= feature.geometry.coordinates[1] &&
              feature.geometry.coordinates[1] <= map.getBounds()._ne.lat
            ) {
              feature.properties.insidescreen = true;
            } else {
              feature.properties.insidescreen = false;
            }
          });

          for (
            var marker = 0;
            marker < this.geojsonobj[solution.name].features.length;
            marker++
          ) {
            document.getElementById("device_list").innerHTML +=
              `<li><a class='device_list_item hover:text-white-snow cursor-pointer' id="list_device_` +
              solution.name +
              `_` +
              marker +
              `">` +
              this.geojsonobj[solution.name].features[marker].properties.name +
              `</a></li>`;
          }
        });
      }
    },

    searchfilter() {

      if(this.screen_width >= 768){

        let i = 0;

        this.solutions.forEach((solution, index) => {
          var input,
            filter,
            ul,
            li,
            Name,
            id,
            textId,
            txtValueName,
            geojsonobj = this.geojsonobj[solution.name];

          //text input
          input = document.getElementById("searchbar");

          //text to filter
          filter = input.value.toUpperCase();

          ul = document.getElementById("device_list");
          li = ul.getElementsByTagName("li");

          geojsonobj.features.forEach((feature) => {
            // id search
            id = feature.properties.id;
            textId = id.toString();

            // name search
            Name = feature.properties.name;
            txtValueName = Name.toString();

            if (filter != "") {
              if (
                txtValueName.toUpperCase().indexOf(filter) > -1 ||
                filter == "ALL" ||
                textId.indexOf(filter) > -1
              ) {
                li[i].style.display = "";
              } else {
                li[i].style.display = "none";
              }
            } else {
              if (feature.properties.insidescreen == true && this.showlist) {
                li[i].style.display = "";
              } else {
                li[i].style.display = "none";
              }
            }

            i++;
          });
        });
      }
    },

    MapFitBounds(timeout, map, bounds) {
      setTimeout(() => {
        //bounds = new mapboxgl.LngLatBounds();
        this.solutions.forEach((solution, index) => {
          this.geojsonobj[solution.name].features.forEach((feature) => {
            bounds.extend(feature.geometry.coordinates);
          });
        });

        map.fitBounds(
          bounds,
          { padding: { top: 90, bottom: 90, left: 160, right: 160 } },
          { duration: 0 }
        );
      }, timeout);
    },

    // gets the coordenates from the device, cluster, etc we are clicking or hovering
    coordinatesFromMarker(e) {
      const coordinates = e.features[0].geometry.coordinates.slice();

      while (Math.abs(e.lngLat.lng - coordinates[0]) > 180) {
        coordinates[0] += e.lngLat.lng > coordinates[0] ? 360 : -360;
      }

      return coordinates;
    },

    show(e, id, status, map) {
      this.hide();
      this.sidemenu_iddevice = id;
      var marker;

      //id
      this.solutions.forEach((solution, index) => {
        this.geojsonobj[solution.name].features.forEach((feature, index) => {
          if (feature.properties.id == id) {
            marker = index;
            if (this.screen_width >= 768 && feature.properties.status == 1)
              this.getfulldata(feature.properties.id);
          }
        });
      });

      if(document.getElementById("sidemenu")){
      document.getElementById("sidemenu").style.width = "18rem";
      document.getElementById("sidemenu").style.padding =
        "0.5rem 0.5rem 0 0.5rem";
      }

      if(document.getElementById("sidemenu_default")){
      document.getElementById("sidemenu_default").style.width = "0";
      document.getElementById("sidemenu_default").style.padding = "0";
      }

      // to point the device in the center of the map (sidemenu open)
      var metersPerPx =
        (156543.03392 * Math.cos((map.getCenter().lat * Math.PI) / 180)) /
        Math.pow(2, map.getZoom());

      if (this.screen_width >= 768 && status == 1) e.lngLat.lng += metersPerPx * 0.0009375;

      map.flyTo({
        center: e.lngLat,
        essential: true,
      });
    },

    hide() {
      this.sidemenu_iddevice = null;

      if(document.getElementById("sidemenu")){
      document.getElementById("sidemenu").style.width = "0";
      document.getElementById("sidemenu").style.padding = "0";
      }

      if(document.getElementById("sidemenu_default")){
      document.getElementById("sidemenu_default").style.width = "18rem";
      document.getElementById("sidemenu_default").style.padding =
        "0 0.5rem 0 0.5rem";
      }

      this.charts = new Array(15);
    },

    getfulldata(id) {

      this.getData(id).then((response) => {

        this.setDeviceData(response, id);

      });
    },

    setDeviceData(response, id) {

      this.charts = new Array(15);
      let highcharts_charts = new Array();

      this.charts_size = new Array();
      this.charts_text = new Array();
      this.charts_height = new Array();
      this.charts_null = new Array();

      document.getElementById("sidemenu_content_before").innerHTML = "";
      document.getElementById("sidemenu_content_after").innerHTML = "";

      if(response.type_name == null){

        window.emitter.emit(
          'AppLayout_alert',
          true,
          this.$t('general.Gateway without records'),
          this.$t('text.lacks records'),
          '16rem'
        )

      } else if (response.type_name == "MODBUS") {

        if(response.MODBUS_device_type == 6){

          let sensor_num;
          let index_i;

          if (response.data) {
            sensor_num = Object.keys(response.data).length;
          } else {
            sensor_num = 1;
          }

          let units = new Array(sensor_num);
          let names = new Array(sensor_num);
          let data = new Array(sensor_num);
          let dates;

          const Highcharts = require("highcharts");
          require('highcharts/highcharts-more')(Highcharts);

          if (document.getElementsByClassName("mapboxgl-popup").length) {
            document.getElementById('popupDate_' + id).innerHTML = response.dates[0];
          }

          for (let sensor in response.data) {

            if(response.data[sensor].length != 1){ // graficos

              switch(sensor){
                case '15': index_i = 0; break;
                case '9': index_i = 1; break;
              }

              this.charts[index_i] = {
                name:
                  (response.units[sensor] != '') ? (response.names[sensor] + " (" + response.units[sensor] + ")") : (response.names[sensor]),
              };

              dates = new Array(response.data[sensor].length);
              data[index_i] = new Array(response.data[sensor].length);
              names[index_i] = response.names[sensor];
              units[index_i] = response.units[sensor];

              response.data[sensor].forEach((value, index) => {
                dates[index] = new Date(
                  (response.dates[index] + " GMT")
                ).getTime();
                data[index_i][index] = parseFloat(value);
              });

            }else if(!(sensor == 11 || sensor == 13 || sensor == 10 || sensor == 12)){ // texto y gauge

              switch(sensor){
                case '14': index_i = 2; break;
                case '16': index_i = 3; break;
                case '17': index_i = 4; break;
                case '1': index_i = 5; break;
                case '3': index_i = 6; break;
                case '6': index_i = 7; break;
                case '7': index_i = 8; break;
                case '8': index_i = 9; break;
              }

              this.charts[index_i] = {
                name: response.names[sensor],
              };

                this.charts_height.push(index_i);
                this.charts_size.push(index_i);

              data[index_i] = response.data[sensor][0];
              names[index_i] = response.names[sensor];
              units[index_i] = response.units[sensor];

            }else if(sensor == 11 || sensor == 13 || sensor == 10 || sensor == 12){ //words

              switch(sensor){
                case '11': index_i = 11; break;
                case '13': index_i = 10; break;
                case '10': index_i = 12; break;
                case '12': index_i = 13; break;
              }

              if(sensor != 10){

                this.charts[index_i] = {
                  name: response.names[sensor],
                };
              }else{

                this.charts[index_i] = {
                  name: "DI1, DI2, DI3",
                };
              }

              if(sensor == 11 || sensor == 10){
                //this.charts_height.push(index_i);
                this.charts_size.push(index_i);
              }

              this.charts_null.push(index_i);

              data[index_i] = response.data[sensor][0];
              names[index_i] = response.names[sensor];
              units[index_i] = response.units[sensor] == null ? '' : response.units[sensor];

            }

          }

          data.forEach((sensor, index) => {

            let Chart = this.Chart;

            if(typeof sensor === 'object'){ //grafico

              let highcharts_data = new Array(sensor.length);

              sensor.forEach((measure, index) => {
                highcharts_data[index] = [dates[index], parseFloat(measure)];
              });

              Chart.yAxis.tickPositions = [
                parseFloat(Math.min(...sensor)),
                parseFloat(Math.max(...sensor)),
              ];

              Chart.tooltip.valueSuffix = units[index];

              Chart.series = [
                {
                  name: names[index],
                  type: "spline",
                  data: highcharts_data.reverse(),
                  color: "#05fff3",
                  lineWidth: 1.5,
                  zIndex: 1,
                  marker: {
                    enabled: false,
                  },
                  tooltip: {
                    valueSuffix: units[index],
                  },
                },
              ];

              highcharts_charts.push(Highcharts.chart("chart_" + index, Chart));

            }else if(index == 2 || index == 3 || index == 4 || index == 5 || index == 6){ //gauge


              let max;

              switch(index){
                default:  max = 100; break;
              }

              let min;

              switch(index){
                  default:  min = 0; break;
              }

              Chart = {
                chart: {
                  type: 'gauge',
                  backgroundColor: "transparent",
                  plotBackgroundColor: null,
                  plotBackgroundImage: null,
                  plotBorderWidth: 0,
                  plotShadow: false,
                  spacing: [0, 0, 0, 0], // Ajusta los márgenes del gráfico según tus necesidades
                  height: 60,
                  marginTop: -20, // Elimina el margen superior del gráfico
                  marginBottom: -40,
                },
                title: null,
                pane: {
                  startAngle: -105,
                  endAngle: 105,
                  background: {
                    backgroundColor: "transparent",
                    borderWidth: 0
                  }
                },
                tooltip: {
                  enabled: false
                },
                xAxis: {
                  title: {
                    text: "",
                    style: {
                      color: "#FFFAFA",
                      fontSize: "12px",
                      fontWeight: "normal",
                      fontStyle: "normal"
                    }
                  },
                  labels: {
                    rotation: -45,
                    style: {
                      color: 'white',
                      fontSize: '10px' // Ajusta el tamaño de la fuente de las etiquetas según tus necesidades
                    }
                  }
                },
                yAxis: {
                  min: min,
                  max: max,
                  tickPosition: 'outside',
                  lineColor: '#05fff3',
                  lineWidth: 1, // Ajusta el ancho de la línea del eje según tus necesidades
                  minorTickPosition: 'outside',
                  tickColor: '#05fff3',
                  minorTickColor: '#05fff3',
                  tickLength: 4, // Ajusta la longitud de las marcas del eje según tus necesidades
                  minorTickWidth: 1,
                  tickWidth: 1,
                  minorTickLength: 3, // Ajusta la longitud de las marcas menores del eje según tus necesidades
                  labels: {
                    distance: 6, // Ajusta la distancia de las etiquetas al eje según tus necesidades
                    rotation: 'auto',
                    style: {
                      color: '#05fff3',
                      fontSize: '8px' // Ajusta el tamaño de la fuente de las etiquetas según tus necesidades
                    }
                  },
                  tickPositions:
                  [
                    min,
                    min + (max-min)/4,
                    min + (max-min)/2,
                    min + 3*(max-min)/4,
                    max
                  ],
                  offset: -10,
                  endOnTick: false
                },
                plotOptions: {
                  gauge: {
                    borderWidth: 0,
                    borderColor: "#05fff3", // Cambia el color del borde de la serie a #05fff3
                    dataLabels: {
                      enabled: false, // Desactiva las etiquetas con el dato y su unidad
                    },
                    pivot: {
                      radius: "4%",
                      backgroundColor: "#fff", // Cambia el color del pivote a blanco
                    },
                    dial: {
                      backgroundColor: "#fff", // Cambia el color de la flecha a blanco
                      radius: "70%", // Ajusta el tamaño de la flecha según tus necesidades
                      baseWidth: 3, // Ajusta el ancho de la base de la flecha según tus necesidades
                      topWidth: 1, // Ajusta el ancho de la punta de la flecha según tus necesidades
                      baseLength: "10%", // Ajusta la longitud de la base de la flecha según tus necesidades
                      rearLength: "-20%", // Ajusta la longitud de la punta de la flecha según tus necesidades
                    }
                  },
                },
                credits: {
                  enabled: false
                },
                exporting: {
                  enabled: false,
                },
                series: [{
                  name: '',
                  data: [{
                    y: parseFloat(sensor),
                  }],
                  tooltip: {
                    valueSuffix: ''
                  }
                }]
              };

              highcharts_charts.push(Highcharts.chart("chart_" + index, Chart));

              var content =
                '<div class="content-between m-0 cursor-default text-xs w-full pb-2">' +
                  '<div class="text-center flex justify-center items-center w-full whitespace-nowrap gap-2 pl-4">' +
                    '<p class="text-2xl font-sevensegment text-white-snow">' + sensor + '</p>' +
                    '<p class="text-white-snow pb-1 self-end text-left mb-0.5">' + units[index] + '</p>' +
                  '</div>' +
                '</div>'

              document.getElementById("chart_text_" + index).innerHTML = content;

            }else if(index == 7 || index == 8 || index == 9){ // textos

              var content = '';
              var icon_class =
                (index === 7 || index === 8) ? 'DK-voltaje' :
                (index === 9) ? 'DK-temperatura' :
                (false) ? 'DK-analogico' :
                'hidden';

              switch(index){

                case 7: case 8: case 9:
                  content +=
                    '<div class="content-between m-0 cursor-default text-xs w-full">' +
                      '<div class="bg-cyan-opa px-2 mb-3 py-2 mt-1 text-center rounded-lg flex justify-between items-center w-full whitespace-nowrap">' +
                        '<div class="h-6 w-6 rounded-full bg-transparent border border-white grid place-items-center justify-self-start">' +
                          '<i class="' + icon_class + '" style="font-size:0.9rem; color:#FFFAFA;"></i>' +
                        '</div>' +
                        '<p class="text-2xl font-sevensegment text-white-snow">' + sensor + '</p>' +
                        '<p class="text-white-snow pb-1 self-end w-4 text-left mb-0.5">' + units[index] + '</p>' +
                      '</div>' +
                    '</div>'
                  break;

              }

              document.getElementById("chart_text_" + index).innerHTML = content;

            } else { //words

              if( index == 10 ){

                let decode_word = [];

                for (let i = sensor.length - 1; i >= 0; i--) {
                  decode_word.push(sensor[i] == 1 ? true : false)
                }

                while(decode_word.length < 16){
                  decode_word.push(false)
                }

                content = '<div class="grid grid-cols-6 m-0 cursor-default text-xs w-full gap-2 pt-1 pb-4">';

                  decode_word.forEach( (bool, index) => {

                    switch (index) {
                        case 3:
                        content += '<div class="col-span-3 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">Rampa 2 activa</div>'
                        break;

                        case 5:
                        content += '<div class="col-span-3 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">Lugar de CTRL remoto 1 activado</div>'
                        break;

                        case 6:
                        content += '<div class="col-span-3 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">Lugar de CTRL remoto 2 activado</div>'
                        break;

                        case 7:
                        content += '<div class="col-span-3 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">Control activo de bus de campo</div>'
                        break;

                        case 8:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">Control activo local</div>'
                        break;

                        case 9:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">Control activo de PC</div>'
                        break;

                        case 10:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">Frecuencias constantes activas</div>'
                        break;

                        default:
                        // Código para el caso por defecto
                        break;
                    }

                  })

                content += '</div>';

              }else if(index == 11) {

                let decode_word = [];

                for (let i = 0; i < sensor.length; i++) {
                    decode_word.push(sensor[i] == 1 ? true : false)
                }

                content = '<div class="grid grid-cols-1 m-0 cursor-default w-full gap-2 pt-1 pb-4 2xl:text-base text-sm">';

                decode_word.forEach( (bool, index) => {

                    switch (index) {
                        case 0:
                        content += '<div class="w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + 'R01</div>'
                        break;

                        case 1:
                        content += '<div class="w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + 'R02</div>'
                        break;

                        case 2:
                        content += '<div class="w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + 'D0</div>'
                        break;

                        default:
                            // Código para el caso por defecto

                        break;
                    }

                })

                content += '</div>';

              }else if(index == 12) {

                let decode_word = [];

                for (let i = 0; i < sensor.length; i++) {
                    decode_word.push(sensor[i] == 1 ? true : false)
                }

                content = '<div class="grid grid-cols-1 m-0 cursor-default w-full gap-2 pt-1 pb-4 2xl:text-base text-sm">';

                decode_word.forEach( (bool, index) => {

                    switch (index) {
                        case 0:
                        content += '<div class="w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + 'DI1</div>'
                        break;

                        case 1:
                        content += '<div class="w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + 'DI2</div>'
                        break;

                        case 2:
                        content += '<div class="w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + 'DI3</div>'
                        break;

                        default:
                            // Código para el caso por defecto

                        break;
                    }

                })

                content += '</div>';

              }else if(index == 13) {

                // B0 = Preparado; B1 = Marcha; B2 = Inverso; B3 = Fallo; B6 = Permiso de marcha; B7 = Alarma activa; B12 = Solicitud de marcha; B13 = Regulador del motor activado

                let decode_word = [];

                for (let i = sensor.toString().length - 1; i >= 0; i--) {
                    decode_word.push(sensor.toString()[i] == 1 ? true : false)
                }

                while(decode_word.length <= 13){
                    decode_word.push(false)
                }

                content = '<div class="grid grid-cols-6 m-0 cursor-default w-full gap-2 pt-1 pb-4 2xl:text-base text-xs">';

                decode_word.forEach( (bool, index) => {

                    switch (index) {
                        case 0:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Ready") + '</div>'
                        break;

                        case 1:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Run") + '</div>'
                        break;

                        case 2:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Reverse") + '</div>'
                        break;

                        case 3:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-fault text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Fault") + '</div>'
                        break;

                        case 6:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-alarm text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Run_request") + '</div>'
                        break;

                        case 7:
                        content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-regulator text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Alarm") + '</div>'
                        break;

                        case 12:
                        content += '<div class="col-span-3 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Run_enable") + '</div>'
                        break;

                        case 13:
                        content += '<div class="col-span-3 w-full text-center rounded-md border border-gray-semimedium py-1.5 ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Regulator") + '</div>'
                        break;

                        default:
                        // Código para el caso por defecto
                        break;
                    }

                })

                content += '</div>';
              }

              setTimeout(() => {
                document.getElementById("chart_text_" + index).innerHTML = content;
              }, 0);

            }

          });

        }else if(response.MODBUS_device_type == 7){

          let sensor_num;
          let index_i;

          if (response.data) {
            sensor_num = Object.keys(response.data).length;
          } else {
            sensor_num = 1;
          }

          let units = new Array(sensor_num);
          let names = new Array(sensor_num);
          let data = new Array(sensor_num);
          let dates;

          const Highcharts = require("highcharts");
          require('highcharts/highcharts-more')(Highcharts);

          if (document.getElementsByClassName("mapboxgl-popup").length) {
            document.getElementById('popupDate_' + id).innerHTML = response.dates[0];
          }

          for (let sensor in response.data) {

            if(response.data[sensor].length != 1 && sensor != 0){

              /*switch(sensor){ //CHARTS
                case '1': index_i = 0; break;
                case '3': index_i = 1; break;
              }*/

              index_i = parseInt(sensor) -1

              this.charts[index_i] = {
                name:
                  (response.units[sensor] != '') ? (response.names[sensor] + " (" + response.units[sensor] + ")") : (response.names[sensor]),
              };

              dates = new Array(response.data[sensor].length);
              data[index_i] = new Array(response.data[sensor].length);
              names[index_i] = response.names[sensor];
              units[index_i] = response.units[sensor];

              response.data[sensor].forEach((value, index) => {
                dates[index] = new Date(
                  (response.dates[index] + " GMT")
                ).getTime();
                data[index_i][index] = parseFloat(value);
              });

              /*switch(sensor){ //Gauge del ultimo dato del chart
                case '1': index_i = 2; break;
                case '3': index_i = 3; break;
              }

              this.charts[index_i] = {
                name: response.names[sensor],
                  //(response.units[sensor] != '') ? (response.names[sensor] + " (" + response.units[sensor] + ")") : (response.names[sensor]),
              };

              this.charts_height.push(index_i);
              this.charts_size.push(index_i);

              data[index_i] = response.data[sensor][0];
              names[index_i] = response.names[sensor];
              units[index_i] = response.units[sensor];*/

            } else if(sensor != 0){ //Gauge y texto

              switch(sensor){
                case '14': index_i = 4; break;
                case '36': index_i = 5; break;
                case '18': index_i = 6; break;
              }


              this.charts[index_i] = {
                name: response.names[sensor],
              };

              this.charts_height.push(index_i);
              this.charts_size.push(index_i);

              data[index_i] = response.data[sensor][0];
              names[index_i] = response.names[sensor];
              units[index_i] = response.units[sensor];

            } else if(sensor == 0){ //WORDS

              switch(sensor){
              }

              this.charts[index_i] = {
                name: response.names[sensor],
              };

              this.charts_null.push(index_i);

              data[index_i] = response.data[sensor][0];
              names[index_i] = response.names[sensor];
              units[index_i] = response.units[sensor] == null ? '' : response.units[sensor];

            }

          }

          data.forEach((sensor, index) => {

            let Chart = this.Chart;

            if(typeof sensor === 'object'){

              let highcharts_data = new Array(sensor.length);

              sensor.forEach((measure, index) => {
                highcharts_data[index] = [dates[index], parseFloat(measure)];
              });

              Chart.yAxis.tickPositions = [
                parseFloat(Math.min(...sensor)),
                parseFloat(Math.max(...sensor)),
              ];

              Chart.tooltip.valueSuffix = units[index];

              Chart.series = [
                {
                  name: names[index],
                  type: "spline",
                  data: highcharts_data.reverse(),
                  color: "#05fff3",
                  lineWidth: 1.5,
                  zIndex: 1,
                  marker: {
                    enabled: false,
                  },
                  tooltip: {
                    valueSuffix: units[index],
                  },
                },
              ];

              highcharts_charts.push(Highcharts.chart("chart_" + index, Chart));

            }else if(false){

              let max;

              switch(index){
                case 0:   max = 1000; break;
                default:  max = 100; break;
              }

              let min;

              switch(index){
                case 0:   min = 0; break;
                default:  min = 0; break;
              }

              Chart = {
                chart: {
                  type: 'gauge',
                  backgroundColor: "transparent",
                  plotBackgroundColor: null,
                  plotBackgroundImage: null,
                  plotBorderWidth: 0,
                  plotShadow: false,
                  spacing: [0, 0, 0, 0], // Ajusta los márgenes del gráfico según tus necesidades
                  height: 60,
                  marginTop: -20, // Elimina el margen superior del gráfico
                  marginBottom: -40,
                },
                title: null,
                pane: {
                  startAngle: -105,
                  endAngle: 105,
                  background: {
                    backgroundColor: "transparent",
                    borderWidth: 0
                  }
                },
                tooltip: {
                  enabled: false
                },
                xAxis: {
                  title: {
                    text: "",
                    style: {
                      color: "#FFFAFA",
                      fontSize: "12px",
                      fontWeight: "normal",
                      fontStyle: "normal"
                    }
                  },
                  labels: {
                    rotation: -45,
                    style: {
                      color: 'white',
                      fontSize: '10px' // Ajusta el tamaño de la fuente de las etiquetas según tus necesidades
                    }
                  }
                },
                yAxis: {
                  min: min,
                  max: max,
                  tickPosition: 'outside',
                  lineColor: '#05fff3',
                  lineWidth: 1, // Ajusta el ancho de la línea del eje según tus necesidades
                  minorTickPosition: 'outside',
                  tickColor: '#05fff3',
                  minorTickColor: '#05fff3',
                  tickLength: 4, // Ajusta la longitud de las marcas del eje según tus necesidades
                  minorTickWidth: 1,
                  tickWidth: 1,
                  minorTickLength: 3, // Ajusta la longitud de las marcas menores del eje según tus necesidades
                  labels: {
                    distance: 6, // Ajusta la distancia de las etiquetas al eje según tus necesidades
                    rotation: 'auto',
                    style: {
                      color: '#05fff3',
                      fontSize: '8px' // Ajusta el tamaño de la fuente de las etiquetas según tus necesidades
                    }
                  },
                  tickPositions: [min, max],
                  offset: -10,
                  endOnTick: false
                },
                plotOptions: {
                  gauge: {
                    borderWidth: 0,
                    borderColor: "#05fff3", // Cambia el color del borde de la serie a #05fff3
                    dataLabels: {
                      enabled: false, // Desactiva las etiquetas con el dato y su unidad
                    },
                    pivot: {
                      radius: "4%",
                      backgroundColor: "#fff", // Cambia el color del pivote a blanco
                    },
                    dial: {
                      backgroundColor: "#fff", // Cambia el color de la flecha a blanco
                      radius: "70%", // Ajusta el tamaño de la flecha según tus necesidades
                      baseWidth: 3, // Ajusta el ancho de la base de la flecha según tus necesidades
                      topWidth: 1, // Ajusta el ancho de la punta de la flecha según tus necesidades
                      baseLength: "10%", // Ajusta la longitud de la base de la flecha según tus necesidades
                      rearLength: "-20%", // Ajusta la longitud de la punta de la flecha según tus necesidades
                    }
                  },
                },
                credits: {
                  enabled: false
                },
                exporting: {
                  enabled: false,
                },
                series: [{
                  name: '',
                  data: [{
                    y: sensor,
                  }],
                  tooltip: {
                    valueSuffix: ''
                  }
                }]
              };

              highcharts_charts.push(Highcharts.chart("chart_" + index, Chart));

              var content =
                '<div class="content-between m-0 cursor-default text-xs w-full pb-2">' +
                  '<div class="text-center flex justify-center items-center w-full whitespace-nowrap gap-2 pl-4">' +
                    '<p class="text-2xl font-sevensegment text-white-snow">' + sensor + '</p>' +
                    '<p class="text-white-snow pb-1 self-end text-left mb-0.5">' + units[index] + '</p>' +
                  '</div>' +
                '</div>'

              document.getElementById("chart_text_" + index).innerHTML = content;

            }else if(false){

              var content = '';
              var icon_class = (index === 6) ? 'DK-temperatura' : 'hidden';

              switch(index){

                case 6:
                  var text = "";
                  switch(sensor){
                    case 0: text = "Parado"; break;
                    case 1: text = "En marcha"; break;
                    case 3: text = "Modo dormir"; break;
                    case 4: text = "En banda muerta"; break;
                    default: text = "";
                  }

                  content +=
                    '<div class="content-between m-0 cursor-default text-xs w-full">' +
                      '<div class="bg-cyan-opa px-2 mb-3 py-2 mt-1 text-center rounded-lg flex justify-between items-center w-full whitespace-nowrap">' +
                        /*'<div class="h-6 w-6 rounded-full bg-transparent border border-white grid place-items-center justify-self-start">' +
                          '<i class="' + icon_class + '" style="font-size:0.9rem; color:#FFFAFA;"></i>' +
                        '</div>' +
                        '<p class="text-2xl font-sevensegment text-white-snow">' + sensor + '</p>' +
                        '<p class="text-white-snow pb-1 self-end w-4 text-left mb-0.5">' + units[index] + '</p>' +*/
                        '<p class="text-white-snow self-center w-full text-center">' + text + '</p>' +
                      '</div>' +
                    '</div>'
                  break;

              }

              document.getElementById("chart_text_" + index).innerHTML = content;

            }else if(false){
              //index == 13 || index == 14 || index == 15 || index == 16
            }else{

              let decode_word = [];

              for (let i = sensor.length - 1; i >= 0; i--) {
                decode_word.push(sensor[i] == 1 ? true : false)
              }

              content = '<div class="grid grid-cols-6 m-0 cursor-default text-sm w-full gap-2 pt-1 pb-4">';

              decode_word.forEach( (bool, index) => {

                switch (index) {

                  case 1:
                    content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Ready") + '</div>'
                    break;

                  case 2:
                    content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Run") + '</div>'
                    break;

                  case 3:
                    content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium ' + (bool ? 'bg-fault text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Fault") + '</div>'
                    break;

                  case 6:
                    content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium ' + (bool ? 'bg-alarm text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Alarm") + '</div>'
                    break;

                  case 7:
                    content += '<div class="col-span-2 w-full text-center rounded-md border border-gray-semimedium ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Run_enable") + '</div>'
                    break;

                  case 12:
                    content += '<div class="col-span-3 w-full text-center rounded-md border border-gray-semimedium ' + (bool ? 'bg-cyan-opa-2 text-white-snow' : 'text-white-snow' ) + '">' + this.$t("general.Run_request") + '</div>'
                    break;

                  default:
                    // Código para el caso por defecto
                    break;
                }

              })

              content += '</div>';

              setTimeout(() => {
                document.getElementById("chart_text_" + index).innerHTML = content;
              }, 0);

            }

          });

        } else {

          let sensor_num;

          if (response.data) {
            sensor_num = Object.keys(response.data).length;
          } else {
            sensor_num = 1;
          }

          let units = new Array(sensor_num);
          let names = new Array(sensor_num);
          let data = new Array(sensor_num);
          let dates;

          const Highcharts = require("highcharts");

          if (document.getElementsByClassName("mapboxgl-popup").length) {
            document.getElementById('popupDate_' + id).innerHTML = response.dates[0];
          }

          for (let sensor in response.data) {
            this.charts[sensor - 1] = {
              name:
                (response.units[sensor] != '') ? (response.names[sensor] + " (" + response.units[sensor] + ")") : (response.names[sensor]),
            };

            dates = new Array(response.data[sensor].length);
            data[sensor - 1] = new Array(response.data[sensor].length);
            names[sensor - 1] = response.names[sensor];
            units[sensor - 1] = response.units[sensor];

            response.data[sensor].forEach((value, index) => {
              dates[index] = new Date(
                (response.dates[index] + " GMT")
              ).getTime();
              data[sensor - 1][index] = parseFloat(value);
            });
          }

          data.forEach((sensor, index) => {

            let Chart = this.Chart;

            let highcharts_data = new Array(sensor.length);

            sensor.forEach((measure, index) => {
              highcharts_data[index] = [dates[index], parseFloat(measure)];
            });

            Chart.yAxis.tickPositions = [
              parseFloat(Math.min(...sensor)),
              parseFloat(Math.max(...sensor)),
            ];

            Chart.tooltip.valueSuffix = units[index];

            Chart.series = [
              {
                name: names[index],
                type: "spline",
                data: highcharts_data.reverse(),
                color: "#05fff3",
                lineWidth: 1.5,
                zIndex: 1,
                marker: {
                  enabled: false,
                },
                tooltip: {
                  valueSuffix: units[index],
                },
              },
            ];

            highcharts_charts.push(Highcharts.chart("chart_" + index, Chart));
          });

        }

      } else if (response.type_name == "PARK") {

        const Highcharts = require("highcharts");

        //sidemenu_content_before

        let days = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];

        let Chart = this.Chart;
        let $t = this.$t;

        //CHART BOOL

        this.charts[0] = {
          name: this.$t("general.Todaysoccupancy"),
        }

        this.charts_height.push(0);

        let highcharts_data_occupied = new Array();

        for (let i = response.epochMS.length - 1; i >= 0; i--) {
            highcharts_data_occupied.push([response.epochMS[i], parseInt(response.data[i].lecture_value)])

            if(i == 0){
                var newDate = new Date();
                const fechaUTC = new Date(newDate.getTime() + ((response.device.hoursOffset + newDate.getTimezoneOffset() / 60) * 60 * 60 * 1000));
                const offsetUTC = fechaUTC.getTimezoneOffset();
                const epochUTC = fechaUTC.getTime() - (offsetUTC * 60 * 1000);
                highcharts_data_occupied.push([epochUTC, parseInt(response.data[0].lecture_value)])
            }
        }

        let highcharts_data_free = new Array(highcharts_data_occupied.length);

        highcharts_data_occupied.forEach((measure, index) => {
            highcharts_data_free[index] = [measure[0], 1 - measure[1]];
        })

        Chart = {
          chart: {
            backgroundColor: "transparent",
            spacingTop: 15,
          },
          accessibility: {
            enabled: false,
          },
          title: {
            text: "",
          },
          xAxis: {
            startOnTick: false,
            minPadding: 0,
            lineColor: "#647F8C",
            visible: true,
            labels: {
              formatter: function () {
                return Highcharts.dateFormat('%kh', this.value);
              },
              style: {
                color: "rgba(200, 200, 200, 0.9)",
                opacity: 0.7,
              },
            },
            type: "datetime",
          },
          yAxis: {
            endOnTick: false,
            visible: false,
          },
          exporting: {
            enabled: false,
          },
          credits: {
            enabled: false,
          },
          legend: {
            enabled: false,
          },
          plotOptions: {
            area: {
              lineWidth: 0,
            },
            stacking: 'normal',
            series: {
              marker: {
                enabled: false,
                states: {
                  hover: {
                    enabled: false
                  }
                }
              }
            }
          },
          series: [
            {
              type: "area",
              name: '',
              data: highcharts_data_occupied,
              color: "#FF0000",
              zIndex: 1,
              step: 'left',
              marker: {
                enabled: false,
                symbol: null, // elige un cuadrado como marcador
                fillColor: null, // color de relleno del marcador
                lineWidth: 0, // grosor del borde del marcador
                lineColor: null // color del borde del marcador
              },
              tooltip: {
                valueSuffix: this.$t("general.Taken-spot"),
              },
            },
            {
              type: "area",
              name: '',
              data: highcharts_data_free,
              color: "#99FF00",
              zIndex: 1,
              step: 'left',
              marker: {
                enabled: false,
                symbol: null, // elige un cuadrado como marcador
                fillColor: null, // color de relleno del marcador
                lineWidth: 0, // grosor del borde del marcador
                lineColor: null // color del borde del marcador
              },
              tooltip: {
                valueSuffix: this.$t("general.Available-spot"),
              },
            },
          ],
          tooltip: {
            valueSuffix: "",
            shared: true,
            split: false,
            backgroundColor: "rgba(7,82,103,0.95)",
            borderRadius: 12,
            borderWidth: 1,
            borderColor: "rgba(250, 250, 250, 0.65)",
            padding: 6,

            formatter: function () {
              const date = new Date(this.x);

              const year = date.getUTCFullYear();
              const month = ('0' + (date.getUTCMonth() + 1)).slice(-2);
              const day = ('0' + date.getUTCDate()).slice(-2);
              const hours = ('0' + date.getUTCHours()).slice(-2);
              const minutes = ('0' + date.getUTCMinutes()).slice(-2);
              const formattedDateTime = `${year}-${month}-${day} ${hours}:${minutes}`;

              var s = '';

              $.each(this.points, function () {

                  if (this.y == 1) {

                      s +=
                          '<p style="font-size:14px; line-height: 14px; font-weight: 400; color:' + this.color + '"> ' +
                          this.series.userOptions.tooltip.valueSuffix +
                          "</p><br/>";
                  }
              });

              s +=
                  "<b>" +
                  '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
                  $t('general.Since') + ' ' +
                  formattedDateTime +
                  "</p><br/>";

              return s;
            },
          },
        };

        highcharts_charts.push(Highcharts.chart('chart_0', Chart));

        const index_0 = highcharts_charts.length - 1;

        setTimeout(() => {
          highcharts_charts[index_0].reflow();
        }, 0);

        //CHART Parkingspacerotation

        this.charts[1] = {
          name:
            this.$t("general.Parkingspacerotation"),
        };

        var dates = Array.from(Array(24).keys()).map(num => parseInt(num));
        var data = new Array(24).fill(0);
        var rotationsSummary = new Array(24).fill(0);
        var occupancySummary = new Array(24).fill(0);

        var todaysDate = new Date().toISOString().slice(0, 19).replace("T", " ");

        var arrayDatesToday = response.dates.filter(function(date) {

            if(date.substring(0, 10) === todaysDate.substring(0, 10)){
              data[parseInt(date.substring(11, 13))] += 1
            }

            return date.substring(0, 10) === todaysDate.substring(0, 10);
        });

        var occupationDurationAvg = (response.summary.data[0].time_total_occupied / response.summary.data[0].occupations >= 0 ) ? this.formatTime(response.summary.data[0].time_total_occupied / response.summary.data[0].occupations, false) : this.formatTime(0, false);

        let day = new Date().getDay();

        if(typeof response.summary.data[0].occupancyMov_avg[day] != 'undefined') {
          Object.keys(response.summary.data[0].occupancyMov_avg[day]).forEach( (hora, index) => {
            rotationsSummary[index] = parseInt(response.summary.data[0].occupancyMov_avg[day][hora]);
          })
        }

        if(typeof response.summary.data[0].occupancyPer_avg[day] != 'undefined') {
          Object.keys(response.summary.data[0].occupancyPer_avg[day]).forEach( (hora, index) => {
            occupancySummary[index] = parseFloat(response.summary.data[0].occupancyPer_avg[day][hora]);
          })
        }

        Chart = {
          chart: {
            backgroundColor: "transparent",
            spacingTop: 15,
          },
          accessibility: {
            enabled: false,
          },
          title: {
            text: "",
          },
          xAxis: {
            tickPositions: [4, 9, 14, 19],
            categories: dates,
            lineColor: "#647F8C",
            visible: true,
            labels: {
              style: {
                color: "rgba(200, 200, 200, 0.9)",
                opacity: 0.7,
              },
            },
            type: "category",
            plotLines: [{
              value: new Date().getHours(),
              color: 'rgba(200, 200, 200, 0.6)',
              width: 1,
              dashStyle: 'Solid',
            }],
          },
          yAxis: {
            endOnTick: false,
            visible: false,
          },
          exporting: {
            enabled: false,
          },
          credits: {
            enabled: false,
          },
          legend: {
            enabled: false,
          },
          plotOptions: {
            column: {
              borderWidth: 0,
              borderRadius: 3,
              pointWidth: 5,
              pointPadding: 0.05,
              groupPadding: 0.1,
            },
          },
          series: [
            {
              name: '',
              type: "areaspline",
              data: rotationsSummary,
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: "",
              },
              states: {
                hover: {
                  enabled: false
                }
              },
              color: {
                linearGradient: {
                  x1: 0,
                  y1: 0,
                  x2: 0,
                  y2: 1
                },
                stops: [
                  [0, "rgba(7,232,157,0.3)"],
                  [1, "rgba(7,232,157,0.1)"]
                ]
              }
            }, {
              name: this.charts[1].name,
              type: "column",
              data: data,
              color: "#05fff3",
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: "",
              },
            }
          ],
          tooltip: {
            enabled: true,
            valueSuffix: "",
            shared: true,
            split: false,
            backgroundColor: "rgba(7,82,103,0.95)",
            borderRadius: 12,
            borderWidth: 1,
            borderColor: "rgba(250, 250, 250, 0.65)",
            padding: 6,

            formatter: function () {
              var s =
                '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
                this.x + 'h ' + $t("general.To") + ' ' + (parseInt(this.x) + 1) % 24 + 'h' +
                "</p><br/>";

              function times_func(y) {
                if (y != 1) return "general.Rotations";
                else return "general.Rotation";
              }

              $.each(this.points, function () {
                if (this.series.name != '') {
                  s +=
                    '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:#05fff3">' +
                    this.y +
                    " " +
                    $t(times_func(this.y)) +
                    this.series.userOptions.tooltip.valueSuffix +
                    "</p><br/>";
                }
              });

              return s;
            },
          },
        };

        highcharts_charts.push(Highcharts.chart('chart_1', Chart));

        var content = '';

        content +=
          '<div class="px-0.5 pb-3 text-center flex justify-center items-baseline">' +
          '<i class="fa-solid fa-car" style="font-size:0.75rem; color:#05fff3;"></i>' +
          '<p class="text-xs text-gray-light">&nbsp;&nbsp;' + this.$t("text.Rotations of the day") + ': ' + arrayDatesToday.length + '</p></div>';

        document.getElementById("chart_text_1").innerHTML = content;

        //CHART PEAKHOURS

        this.charts[2] = {
          name:
            this.$t("general.PeakHours") + ' (' + this.$t("general." + days[this.week_day % 7] + "_full") + ')',
        };

        Chart = {
          chart: {
            backgroundColor: "transparent",
            spacingTop: 15,
          },
          accessibility: {
            enabled: false,
          },
          title: {
            text: "",
          },
          xAxis: {
            tickPositions: [4, 9, 14, 19],
            categories: dates,
            lineColor: "#647F8C",
            visible: true,
            labels: {
              style: {
                color: "rgba(200, 200, 200, 0.9)",
                opacity: 0.7,
              },
            },
            type: "category",
            plotLines: [{
              value: new Date().getHours(),
              color: 'rgba(200, 200, 200, 0.6)',
              width: 1,
              dashStyle: 'Solid',
            }],
          },
          yAxis: {
            endOnTick: true,
            visible: false,
            tickPositions: [0, 100],
            labels: {
              enabled: true,
            },
            showLastLabel: true,
          },
          exporting: {
            enabled: false,
          },
          credits: {
            enabled: false,
          },
          legend: {
            enabled: false,
          },
          plotOptions: {
            column: {
              borderWidth: 0,
              borderRadius: 3,
              pointWidth: 5,
              pointPadding: 0.05,
              groupPadding: 0.1,
            },
          },
          series: [
            {
              name: this.charts[2].name,
              type: "column",
              data: occupancySummary,
              zones: [
                {
                  value: 80,
                  color: "#05fff3",
                },
                {
                  value: 95,
                  color: "#05fff3",
                },
                {
                  color: "#05fff3",
                },
              ],
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: "",
              },
            },
          ],
          tooltip: {
            enabled: false,
            valueSuffix: "",
            shared: true,
            split: false,
            backgroundColor: "rgba(7,82,103,0.95)",
            borderRadius: 12,
            borderWidth: 1,
            borderColor: "rgba(250, 250, 250, 0.65)",
            padding: 6,

            formatter: function () {
              var s =
                "<b>" +
                '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
                this.x +
                "</p><br/>";
              $.each(this.points, function () {
                s +=
                  '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:#05fff3">' +
                  this.series.name +
                  ':</p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:#05fff3">' +
                  this.y +
                  "min" +
                  this.series.userOptions.tooltip.valueSuffix +
                  "</p><br/>";
              });

              return s;
            },
          },
        };

        highcharts_charts.push(Highcharts.chart("chart_2", Chart));

        content =
          '<div class="px-0.5 pb-3 text-center flex justify-center items-baseline">' +
          '<i class="fa-regular fa-clock" style="font-size:0.75rem; color:#05fff3;"></i>' +
          '<p class="text-xs text-gray-light">&nbsp;&nbsp;' + this.$t("text.Average parking time") + ': ' +
          occupationDurationAvg + '</p></div>';

        document.getElementById("chart_text_2").innerHTML = content;

      } else if (response.type_name == "PULSES"){

        const Highcharts = require("highcharts");
        require('highcharts/highcharts-more')(Highcharts);

        let sensor_num;

        if (response.data) {
          sensor_num = Object.keys(response.data).length;
        } else {
          sensor_num = 0;
        }

        let units = new Array(sensor_num);
        let names = new Array(sensor_num);
        let data = new Array(sensor_num);
        let dates;
        let index = 0;

        if (document.getElementsByClassName("mapboxgl-popup").length && document.getElementById("popupDate_" + id)) {
          document.getElementById("popupDate_" + id).innerHTML = response.dates[0];
          //document.getElementById("popupLastConsumption_"+ id).innerHTML = response.data[2][0];
        }

        for (let sensor in response.data) {

          this.charts[index] = {
                name:
                  (response.units[sensor] != '') ? (response.names[sensor] + " (" + response.units[sensor] + ")") : (response.names[sensor]),
          };

          dates = new Array(response.data[sensor].length);
          data[index] = new Array(response.data[sensor].length);
          names[index] = response.names[sensor];
          units[index] = response.units[sensor];

          response.data[sensor].forEach((value, subindex) => {
            dates[subindex] = new Date(
              (response.dates[subindex] + " GMT")
            ).getTime();
            data[index][subindex] = parseFloat(value);
          });

        index ++;
        }

        data.forEach((sensor, index) => {
          let Chart = this.Chart;

          let highcharts_data = new Array(sensor.length);

          sensor.forEach((measure, index) => {
            highcharts_data[index] = [dates[index], parseFloat(measure)];
          });

          Chart.yAxis.tickPositions = [
            parseFloat(Math.min(...sensor)),
            parseFloat(Math.max(...sensor)),
          ];

              //Chart.xAxis.tickPositions = [0, 23];

          Chart.tooltip.valueSuffix = units[index];

          Chart.series = [
            {
              name: names[index],
              type: "spline",
              data: highcharts_data.reverse(),
              color: "#05fff3",
              lineWidth: 1.5,
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: units[index],
              },
            },
          ];

          highcharts_charts.push(Highcharts.chart("chart_" + index, Chart));

        })
      }

      setTimeout(() => {

        for (let i_chart = 0; i_chart < highcharts_charts.length; i_chart++) {
          if (highcharts_charts[i_chart] != undefined) {
            highcharts_charts[i_chart].reflow();
          }
        }
      }, 200);
    },

    Notify(title, message, time){

      if(time == undefined){
        time = 7500;
      }else{
        time = time * 1000
      }

      this.Show_Notification = true;
      this.Notification_Title = title;
      this.Notification_Message = message;

      setTimeout(() => {
        this.Show_Notification = false;
        this.Notification_Title = "";
        this.Notification_Message = "";
      }, time);

    },
  },
};
</script>



